"""
From: https://github.com/SWE-Gym/SWE-Gym/blob/main/scripts/modal_misc/download_checkpoint.py

Download a checkpoint from Hugging Face.

modal run download_checkpoint.py --source-repo /path/to/source_repo --target-dir /path/to/target_dir

Example:
modal run download_checkpoint.py --source-repo meta-llama/Llama-3.3-70B-Instruct --target-dir /weights/meta-llama/Llama-3.3-70B-Instruct
"""

import modal
import os

app = modal.App("download-hf-ckpts")
model_volume = modal.Volume.from_name("weights", create_if_missing=True)

image = (
    modal.Image.debian_slim(python_version="3.12")
    .apt_install(["git", "git-lfs"])
    .pip_install("huggingface_hub[cli]")
)


MINUTES = 60  # seconds
HOURS = 60 * MINUTES


@app.function(
    volumes={"/weights": model_volume},
    image=image,
    timeout=1 * HOURS,
    secrets=[modal.Secret.from_name("john-hf-secret")],
)
def download_ckpts(source_repo: str, target_dir: str):
    # make sure target_dir exists
    os.makedirs(target_dir, exist_ok=True)

    import subprocess
    import sys

    command = "git lfs install"
    subprocess.run(
        command.split(),
        stdout=sys.stdout,
        stderr=sys.stderr,
        check=True,
    )

    command = f"huggingface-cli download {source_repo} --local-dir {target_dir}"
    subprocess.run(
        command.split(),
        stdout=sys.stdout,
        stderr=sys.stderr,
        check=True,
    )
    model_volume.commit()


@app.local_entrypoint()
def main(source_repo: str, target_dir: str):
    download_ckpts.remote(source_repo=source_repo, target_dir=target_dir)



"""
Purpose: Standalone script to download all SWEFT images

Usage: python -m swesmith.build_repo.download_images
"""

import argparse
import docker
import os
import json
import requests

from swesmith.constants import ORG_NAME_DH

TAG = "latest"


def get_docker_hub_login():
    docker_config_path = os.path.expanduser("~/.docker/config.json")

    try:
        with open(docker_config_path, "r") as config_file:
            docker_config = json.load(config_file)

        auths = docker_config.get("auths", {})
        docker_hub = auths.get("https://index.docker.io/v1/")

        if not docker_hub:
            raise Exception(
                "Docker Hub credentials not found. Please log in using 'docker login'."
            )

        # The token is encoded in Base64 (username:password), decode it
        from base64 import b64decode

        auth_token = docker_hub.get("auth")
        if not auth_token:
            raise Exception("No auth token found in Docker config.")

        decoded_auth = b64decode(auth_token).decode("utf-8")
        username, password = decoded_auth.split(":", 1)
        return username, password

    except FileNotFoundError:
        raise Exception(
            "Docker config file not found. Have you logged in using 'docker login'?"
        )
    except Exception as e:
        raise Exception(f"Error retrieving Docker Hub token: {e}")


def get_dockerhub_token(username, password):
    """Get DockerHub authentication token"""
    auth_url = "https://hub.docker.com/v2/users/login"
    auth_data = {"username": username, "password": password}
    response = requests.post(auth_url, json=auth_data)
    response.raise_for_status()
    return response.json()["token"]


def get_docker_repositories(username, token):
    url = f"https://hub.docker.com/v2/repositories/{username}/"
    headers = {"Authorization": f"Bearer {token}"}

    repositories = []
    while url:
        response = requests.get(url, headers=headers)
        if response.status_code != 200:
            raise Exception(
                f"Failed to fetch repositories: {response.status_code}, {response.text}"
            )

        data = response.json()
        repositories.extend(data.get("results", []))
        url = data.get("next")  # Get the next page URL, if any

    return repositories


def main(repo: str, proceed: bool = True):
    username, password = get_docker_hub_login()
    token = get_dockerhub_token(username, password)
    client = docker.from_env()

    # Get list of swesmith repositories
    repos = get_docker_repositories(ORG_NAME_DH, token)
    repos = [r for r in repos if r["name"].startswith("swesmith")]
    if repo:
        repos = [
            r
            for r in repos
            if repo.replace("__", "_1776_") in r["name"]
            or repo in r["name"]
            or repo.replace("/", "_1776_") in r["name"]
        ]
        if len(repos) == 0:
            print(f"Could not find image for {repo}, exiting...")
            return

    print(f"Found {len(repos)} environments:")
    for idx, r in enumerate(repos):
        print("-", r["name"])
        if idx == 4:
            print(f"(+ {len(repos) - 5} more...)")
            break
    if not proceed and input("Proceed with downloading images? (y/n): ").lower() != "y":
        return

    # Download images
    for r in repos:
        print(f"Downloading {r['name']}...")
        client.images.pull(f"{ORG_NAME_DH}/{r['name']}:{TAG}")


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("--repo", type=str, help="Repository name", default=None)
    parser.add_argument(
        "-y",
        "--proceed",
        action="store_true",
        help="Proceed with downloading images",
    )
    args = parser.parse_args()
    main(**vars(args))



"""
A terminal-based viewer for issue generation results using Textual.
"""

from textual.app import App, ComposeResult
from textual.containers import ScrollableContainer
from textual.widgets import Header, Footer, Static
from textual.binding import Binding
from pathlib import Path
import json
from typing import Any
from rich.markup import escape


class MessageView(Static):
    """A widget to display formatted messages."""

    def __init__(self, problem_statement: str, messages: list):
        content = self._format_content(problem_statement, messages)
        super().__init__(content, markup=False)

    def _format_content(self, problem_statement: str, messages: list) -> str:
        formatted = []
        # Add problem statement with explicit markup
        formatted.append("### Problem Statement ###")
        formatted.append(escape(problem_statement))
        formatted.append("─" * 80)
        formatted.append("\n### Messages ###")

        # Add messages
        for msg in messages:
            role = msg.get("role", "unknown")
            content = msg.get("content", "")
            formatted.append(f"{role.upper()}:")
            formatted.append(escape(content))
            formatted.append("─" * 80)
        return "\n".join(formatted)


class IssueViewer(App):
    """Main application for viewing issue generation results."""

    CSS = """
    #content {
        height: 1fr;  /* Take up remaining space */
        padding: 1 2;
        background: $surface;
        border: solid $primary;
        margin: 1 2;
        overflow-y: scroll;
    }
    """

    BINDINGS = [
        Binding("q", "quit", "Quit"),
        Binding("h", "prev_folder", "Previous Folder"),
        Binding("l", "next_folder", "Next Folder"),
        Binding("j", "scroll_down", "Scroll Down"),
        Binding("k", "scroll_up", "Scroll Up"),
    ]

    def __init__(self, root_dir: str):
        super().__init__()
        self.root_dir = Path(root_dir)
        self.folders: list[Path] = []
        self.current_index = 0
        self._find_valid_folders()

    def _find_valid_folders(self) -> None:
        """Find all folders that contain both messages.json and metadata.json recursively."""

        def is_valid_folder(path: Path) -> bool:
            return (path / "messages.json").exists() and (
                path / "metadata.json"
            ).exists()

        def search_recursively(path: Path) -> None:
            if is_valid_folder(path):
                self.folders.append(path)

            # Search in subdirectories
            for item in path.iterdir():
                if item.is_dir():
                    search_recursively(item)

        # Start recursive search from root directory
        search_recursively(self.root_dir)
        self.folders.sort()

    def _load_data(self, folder: Path) -> tuple[list[dict[str, Any]], dict[str, Any]]:
        """Load messages and metadata from a folder."""
        with open(folder / "messages.json") as f:
            messages = json.load(f)
        with open(folder / "metadata.json") as f:
            metadata = json.load(f)
        return messages, metadata

    def compose(self) -> ComposeResult:
        """Create child widgets for the app."""
        yield Header(show_clock=True)
        yield ScrollableContainer(Static(""), id="content")
        yield Footer()

    def on_mount(self) -> None:
        """Handle app start-up."""
        self.update_view()

    def update_view(self) -> None:
        """Update the display with current folder data."""
        if not self.folders:
            self.query_one("#content").remove_children()
            self.query_one("#content").mount(Static("No valid folders found!"))
            return

        current_folder = self.folders[self.current_index]
        messages, metadata = self._load_data(current_folder)

        # Update title
        self.title = f"Folder: {current_folder.name} [{self.current_index + 1}/{len(self.folders)}]"

        # Get problem statement
        problem = metadata.get("responses", {}).get(
            "problem_statement", "No problem statement found"
        )

        # Create combined view
        content_widget = MessageView(problem, messages)
        self.query_one("#content").remove_children()
        self.query_one("#content").mount(content_widget)

    def action_prev_folder(self) -> None:
        """Handle previous folder action."""
        if self.folders:
            self.current_index = (self.current_index - 1) % len(self.folders)
            self.update_view()
            # Reset scroll position
            self.query_one("#content").scroll_y = 0

    def action_next_folder(self) -> None:
        """Handle next folder action."""
        if self.folders:
            self.current_index = (self.current_index + 1) % len(self.folders)
            self.update_view()
            # Reset scroll position
            self.query_one("#content").scroll_y = 0

    def action_scroll_down(self) -> None:
        """Scroll content down."""
        content = self.query_one("#content")
        content.scroll_y += 10

    def action_scroll_up(self) -> None:
        """Scroll content up."""
        content = self.query_one("#content")
        content.scroll_y = max(0, content.scroll_y - 10)


def main() -> None:
    """Entry point for the viewer."""
    import sys

    if len(sys.argv) != 2:
        print("Usage: python viewer.py <root_directory>")
        sys.exit(1)

    app = IssueViewer(sys.argv[1])
    app.run()


if __name__ == "__main__":
    main()



import tree_sitter_rust as tsrs

from swesmith.bug_gen.procedural.base import CommonPMs
from swesmith.bug_gen.procedural.rust.base import RustProceduralModifier
from swesmith.constants import BugRewrite, CodeEntity
from tree_sitter import Language, Parser

RUST_LANGUAGE = Language(tsrs.language())


class RemoveLoopModifier(RustProceduralModifier):
    explanation: str = CommonPMs.REMOVE_LOOP.explanation
    name: str = CommonPMs.REMOVE_LOOP.name
    conditions: list = CommonPMs.REMOVE_LOOP.conditions

    def modify(self, code_entity: CodeEntity) -> BugRewrite:
        """Remove loop statements from the Rust code."""
        if not self.flip():
            return None

        parser = Parser(RUST_LANGUAGE)
        tree = parser.parse(bytes(code_entity.src_code, "utf8"))

        modified_code = self._remove_loops(code_entity.src_code, tree.root_node)

        if modified_code == code_entity.src_code:
            return None

        return BugRewrite(
            rewrite=modified_code,
            explanation=self.explanation,
            strategy=self.name,
        )

    def _remove_loops(self, source_code: str, node) -> str:
        """Recursively find and remove loop statements."""
        removals = []

        def collect_loops(n):
            if n.type in ["for_expression", "while_expression", "loop_expression"]:
                if self.flip():
                    removals.append(n)
            for child in n.children:
                collect_loops(child)

        collect_loops(node)

        if not removals:
            return source_code

        modified_source = source_code
        for loop_node in reversed(removals):
            start_byte = loop_node.start_byte
            end_byte = loop_node.end_byte

            modified_source = modified_source[:start_byte] + modified_source[end_byte:]

        return modified_source


class RemoveConditionalModifier(RustProceduralModifier):
    explanation: str = CommonPMs.REMOVE_CONDITIONAL.explanation
    name: str = CommonPMs.REMOVE_CONDITIONAL.name
    conditions: list = CommonPMs.REMOVE_CONDITIONAL.conditions

    def modify(self, code_entity: CodeEntity) -> BugRewrite:
        """Remove conditional statements from the Rust code."""
        if not self.flip():
            return None

        parser = Parser(RUST_LANGUAGE)
        tree = parser.parse(bytes(code_entity.src_code, "utf8"))

        modified_code = self._remove_conditionals(code_entity.src_code, tree.root_node)

        if modified_code == code_entity.src_code:
            return None

        return BugRewrite(
            rewrite=modified_code,
            explanation=self.explanation,
            strategy=self.name,
        )

    def _remove_conditionals(self, source_code: str, node) -> str:
        """Recursively find and remove conditional statements."""
        removals = []

        def collect_conditionals(n):
            if n.type == "if_expression":
                if self.flip():
                    removals.append(n)
            for child in n.children:
                collect_conditionals(child)

        collect_conditionals(node)

        if not removals:
            return source_code

        modified_source = source_code
        for if_node in reversed(removals):
            start_byte = if_node.start_byte
            end_byte = if_node.end_byte

            modified_source = modified_source[:start_byte] + modified_source[end_byte:]

        return modified_source


class RemoveAssignModifier(RustProceduralModifier):
    explanation: str = CommonPMs.REMOVE_ASSIGNMENT.explanation
    name: str = CommonPMs.REMOVE_ASSIGNMENT.name
    conditions: list = CommonPMs.REMOVE_ASSIGNMENT.conditions

    def modify(self, code_entity: CodeEntity) -> BugRewrite:
        """Remove assignment statements from the Rust code."""
        if not self.flip():
            return None

        parser = Parser(RUST_LANGUAGE)
        tree = parser.parse(bytes(code_entity.src_code, "utf8"))

        modified_code = self._remove_assignments(code_entity.src_code, tree.root_node)

        if modified_code == code_entity.src_code:
            return None

        return BugRewrite(
            rewrite=modified_code,
            explanation=self.explanation,
            strategy=self.name,
        )

    def _remove_assignments(self, source_code: str, node) -> str:
        """Recursively find and remove assignment statements."""
        removals = []

        def collect_assignments(n):
            if n.type in ["let_declaration", "assignment_expression"]:
                if self.flip():
                    removals.append(n)
            for child in n.children:
                collect_assignments(child)

        collect_assignments(node)

        if not removals:
            return source_code

        modified_source = source_code
        for assign_node in reversed(removals):
            start_byte = assign_node.start_byte
            end_byte = assign_node.end_byte

            modified_source = modified_source[:start_byte] + modified_source[end_byte:]

        return modified_source



"""
JavaScript operation modifiers for procedural bug generation using tree-sitter.
"""

import sys
import tree_sitter_javascript as tsjs
from swesmith.bug_gen.procedural.javascript.base import JavaScriptProceduralModifier
from swesmith.bug_gen.procedural.base import CommonPMs
from swesmith.constants import CodeProperty, BugRewrite, CodeEntity
from tree_sitter import Language, Parser

JS_LANGUAGE = Language(tsjs.language())


def _safe_decode(bytes_obj, fallback=""):
    """Safely decode bytes to UTF-8, handling potential encoding errors."""
    try:
        return bytes_obj.decode("utf-8")
    except UnicodeDecodeError as e:
        print(f"WARNING: UTF-8 decode error: {e}", file=sys.stderr)
        return fallback


class OperationChangeModifier(JavaScriptProceduralModifier):
    """Change operators within similar groups (e.g., +/-, *//%, etc.)"""

    explanation: str = CommonPMs.OPERATION_CHANGE.explanation
    name: str = CommonPMs.OPERATION_CHANGE.name
    conditions: list = CommonPMs.OPERATION_CHANGE.conditions

    def modify(self, code_entity: CodeEntity) -> BugRewrite:
        """Change operators to others in their group."""

        parser = Parser(JS_LANGUAGE)
        tree = parser.parse(bytes(code_entity.src_code, "utf8"))

        modified_code = self._change_operators(code_entity.src_code, tree.root_node)

        if modified_code == code_entity.src_code:
            return None

        return BugRewrite(
            rewrite=modified_code,
            explanation=self.explanation,
            strategy=self.name,
        )

    def _change_operators(self, source_code: str, node) -> str:
        """Find and change binary operators within their groups."""
        changes = []

        # Operator groups
        operator_groups = {
            "+": ["+", "-"],
            "-": ["+", "-"],
            "*": ["*", "/", "%"],
            "/": ["*", "/", "%"],
            "%": ["*", "/", "%"],
            "&": ["&", "|", "^"],
            "|": ["&", "|", "^"],
            "^": ["&", "|", "^"],
            "<<": ["<<", ">>"],
            ">>": ["<<", ">>"],
        }

        def collect_binary_ops(n):
            if n.type == "binary_expression":
                # Find the operator child
                for child in n.children:
                    if child.type in operator_groups:
                        operator = child.type
                        group = operator_groups[operator]
                        # Choose a different operator from the group
                        other_ops = [op for op in group if op != operator]
                        if other_ops and self.flip():
                            new_op = self.rand.choice(other_ops)
                            changes.append({"node": child, "new_op": new_op})
                        break

            for child in n.children:
                collect_binary_ops(child)

        collect_binary_ops(node)

        if not changes:
            return source_code

        # Work with bytes for modifications
        modified_source = source_code.encode("utf-8")
        for change in reversed(changes):
            node = change["node"]
            new_op = change["new_op"]

            modified_source = (
                modified_source[: node.start_byte]
                + new_op.encode("utf-8")
                + modified_source[node.end_byte :]
            )

        return _safe_decode(modified_source, source_code)


class OperationFlipOperatorModifier(JavaScriptProceduralModifier):
    """Flip operators to their opposites (e.g., == to !=, < to >, etc.)"""

    explanation: str = "The operators in an expression are likely incorrect."
    name: str = "func_pm_op_flip"
    conditions: list = [CodeProperty.IS_FUNCTION, CodeProperty.HAS_BINARY_OP]

    def modify(self, code_entity: CodeEntity) -> BugRewrite:
        """Flip operators to their opposites."""

        parser = Parser(JS_LANGUAGE)
        tree = parser.parse(bytes(code_entity.src_code, "utf8"))

        modified_code = self._flip_operators(code_entity.src_code, tree.root_node)

        if modified_code == code_entity.src_code:
            return None

        return BugRewrite(
            rewrite=modified_code,
            explanation=self.explanation,
            strategy=self.name,
        )

    def _flip_operators(self, source_code: str, node) -> str:
        """Find and flip binary operators to their opposites."""
        changes = []

        operator_flips = {
            "===": "!==",
            "!==": "===",
            "==": "!=",
            "!=": "==",
            "<=": ">",
            ">=": "<",
            "<": ">=",
            ">": "<=",
            "&&": "||",
            "||": "&&",
            "+": "-",
            "-": "+",
            "*": "/",
            "/": "*",
        }

        def collect_binary_ops(n):
            if n.type == "binary_expression":
                # Find the operator child
                for child in n.children:
                    if child.type in operator_flips:
                        if self.flip():
                            changes.append(
                                {"node": child, "new_op": operator_flips[child.type]}
                            )
                        break

            for child in n.children:
                collect_binary_ops(child)

        collect_binary_ops(node)

        if not changes:
            return source_code

        # Work with bytes for modifications
        modified_source = source_code.encode("utf-8")
        for change in reversed(changes):
            node = change["node"]
            new_op = change["new_op"]

            modified_source = (
                modified_source[: node.start_byte]
                + new_op.encode("utf-8")
                + modified_source[node.end_byte :]
            )

        return _safe_decode(modified_source, source_code)


class OperationSwapOperandsModifier(JavaScriptProceduralModifier):
    """Swap operands in binary operations (e.g., a + b becomes b + a)"""

    explanation: str = CommonPMs.OPERATION_SWAP_OPERANDS.explanation
    name: str = CommonPMs.OPERATION_SWAP_OPERANDS.name
    conditions: list = CommonPMs.OPERATION_SWAP_OPERANDS.conditions

    def modify(self, code_entity: CodeEntity) -> BugRewrite:
        """Swap left and right operands."""

        parser = Parser(JS_LANGUAGE)
        tree = parser.parse(bytes(code_entity.src_code, "utf8"))

        modified_code = self._swap_operands(code_entity.src_code, tree.root_node)

        if modified_code == code_entity.src_code:
            return None

        return BugRewrite(
            rewrite=modified_code,
            explanation=self.explanation,
            strategy=self.name,
        )

    def _swap_operands(self, source_code: str, node) -> str:
        """Find and swap operands in binary expressions."""
        changes = []
        source_bytes = source_code.encode("utf-8")

        def collect_binary_ops(n):
            if n.type == "binary_expression" and len(n.children) >= 3:
                # Binary expression has: left, operator, right
                left = n.children[0]
                operator_node = n.children[1]
                right = n.children[2]

                if self.flip():
                    # For comparison operators, we might need to flip them too
                    operator = operator_node.type
                    if operator in ["<", ">", "<=", ">="]:
                        op_flip = {"<": ">", ">": "<", "<=": ">=", ">=": "<="}
                        operator = op_flip.get(operator, operator)

                    changes.append(
                        {
                            "node": n,
                            "left": left,
                            "right": right,
                            "operator": operator,
                        }
                    )

            for child in n.children:
                collect_binary_ops(child)

        collect_binary_ops(node)

        if not changes:
            return source_code

        # Work with bytes for modifications
        modified_source = source_bytes
        for change in reversed(changes):
            node = change["node"]
            left = change["left"]
            right = change["right"]
            operator = change["operator"]

            left_text = _safe_decode(source_bytes[left.start_byte : left.end_byte])
            right_text = _safe_decode(source_bytes[right.start_byte : right.end_byte])

            # Swap: left op right -> right op left
            swapped = f"{right_text} {operator} {left_text}"

            modified_source = (
                modified_source[: node.start_byte]
                + swapped.encode("utf-8")
                + modified_source[node.end_byte :]
            )

        return _safe_decode(modified_source, source_code)


class OperationChangeConstantsModifier(JavaScriptProceduralModifier):
    """Change numeric constants to introduce off-by-one errors"""

    explanation: str = CommonPMs.OPERATION_CHANGE_CONSTANTS.explanation
    name: str = CommonPMs.OPERATION_CHANGE_CONSTANTS.name
    conditions: list = CommonPMs.OPERATION_CHANGE_CONSTANTS.conditions

    def modify(self, code_entity: CodeEntity) -> BugRewrite:
        """Change constants by small amounts."""

        parser = Parser(JS_LANGUAGE)
        tree = parser.parse(bytes(code_entity.src_code, "utf8"))

        modified_code = self._change_constants(code_entity.src_code, tree.root_node)

        if modified_code == code_entity.src_code:
            return None

        return BugRewrite(
            rewrite=modified_code,
            explanation=self.explanation,
            strategy=self.name,
        )

    def _change_constants(self, source_code: str, node) -> str:
        """Find and change numeric constants."""
        changes = []
        source_bytes = source_code.encode("utf-8")

        def collect_numbers(n):
            if n.type == "number":
                if self.flip():
                    try:
                        value_text = _safe_decode(
                            source_bytes[n.start_byte : n.end_byte]
                        )
                        value = int(value_text)
                        # Small off-by-one changes
                        new_value = value + self.rand.choice([-1, 1, -2, 2])
                        changes.append({"node": n, "new_value": str(new_value)})
                    except ValueError:
                        pass  # Skip floats and hex numbers

            for child in n.children:
                collect_numbers(child)

        collect_numbers(node)

        if not changes:
            return source_code

        # Work with bytes for modifications
        modified_source = source_bytes
        for change in reversed(changes):
            node = change["node"]
            new_value = change["new_value"]

            modified_source = (
                modified_source[: node.start_byte]
                + new_value.encode("utf-8")
                + modified_source[node.end_byte :]
            )

        return _safe_decode(modified_source, source_code)


class OperationBreakChainsModifier(JavaScriptProceduralModifier):
    """Break chained operations by removing parts of the chain"""

    explanation: str = CommonPMs.OPERATION_BREAK_CHAINS.explanation
    name: str = CommonPMs.OPERATION_BREAK_CHAINS.name
    conditions: list = CommonPMs.OPERATION_BREAK_CHAINS.conditions

    def modify(self, code_entity: CodeEntity) -> BugRewrite:
        """Break chained binary operations."""

        parser = Parser(JS_LANGUAGE)
        tree = parser.parse(bytes(code_entity.src_code, "utf8"))

        modified_code = self._break_chains(code_entity.src_code, tree.root_node)

        if modified_code == code_entity.src_code:
            return None

        return BugRewrite(
            rewrite=modified_code,
            explanation=self.explanation,
            strategy=self.name,
        )

    def _break_chains(self, source_code: str, node) -> str:
        """Find and break chained operations."""
        changes = []
        source_bytes = source_code.encode("utf-8")

        def collect_chains(n):
            if n.type == "binary_expression" and len(n.children) >= 3:
                left = n.children[0]
                operator = n.children[1]
                right = n.children[2]

                # Check if left or right is also a binary expression (chain)
                if left.type == "binary_expression" and self.flip():
                    # Break left chain: keep only the right part
                    # (a + b) + c -> b + c (take right of left chain)
                    if len(left.children) >= 3:
                        left_right = left.children[2]
                        left_right_text = _safe_decode(
                            source_bytes[left_right.start_byte : left_right.end_byte]
                        )
                        operator_text = _safe_decode(
                            source_bytes[operator.start_byte : operator.end_byte]
                        )
                        right_text = _safe_decode(
                            source_bytes[right.start_byte : right.end_byte]
                        )
                        changes.append(
                            {
                                "node": n,
                                "replacement": f"{left_right_text} {operator_text} {right_text}",
                            }
                        )

                elif right.type == "binary_expression" and self.flip():
                    # Break right chain: keep only the left part
                    # a + (b + c) -> a + b (take left of right chain)
                    if len(right.children) >= 3:
                        right_left = right.children[0]
                        left_text = _safe_decode(
                            source_bytes[left.start_byte : left.end_byte]
                        )
                        operator_text = _safe_decode(
                            source_bytes[operator.start_byte : operator.end_byte]
                        )
                        right_left_text = _safe_decode(
                            source_bytes[right_left.start_byte : right_left.end_byte]
                        )
                        changes.append(
                            {
                                "node": n,
                                "replacement": f"{left_text} {operator_text} {right_left_text}",
                            }
                        )

            for child in n.children:
                collect_chains(child)

        collect_chains(node)

        if not changes:
            return source_code

        # Work with bytes for modifications
        modified_source = source_bytes
        for change in reversed(changes):
            node = change["node"]
            replacement = change["replacement"]

            modified_source = (
                modified_source[: node.start_byte]
                + replacement.encode("utf-8")
                + modified_source[node.end_byte :]
            )

        return _safe_decode(modified_source, source_code)


class AugmentedAssignmentSwapModifier(JavaScriptProceduralModifier):
    """Swap augmented assignment operators (+=, -=, *=, /=, etc.) and update expressions (++, --)"""

    explanation: str = (
        "The augmented assignment or update operator is likely incorrect."
    )
    name: str = "func_pm_aug_assign_swap"
    conditions: list = [CodeProperty.IS_FUNCTION, CodeProperty.HAS_ASSIGNMENT]

    def modify(self, code_entity: CodeEntity) -> BugRewrite:
        """Swap augmented assignment operators."""

        parser = Parser(JS_LANGUAGE)
        tree = parser.parse(bytes(code_entity.src_code, "utf8"))

        modified_code = self._swap_augmented_assignments(
            code_entity.src_code, tree.root_node
        )

        if modified_code == code_entity.src_code:
            return None

        return BugRewrite(
            rewrite=modified_code,
            explanation=self.explanation,
            strategy=self.name,
        )

    def _swap_augmented_assignments(self, source_code: str, node) -> str:
        """Find and swap augmented assignment operators and update expressions."""
        changes = []
        source_bytes = source_code.encode("utf-8")

        # Augmented assignment operator swap pairs
        aug_assign_swaps = {
            # Arithmetic
            "+=": "-=",
            "-=": "+=",
            "*=": "/=",
            "/=": "*=",
            "%=": "/=",
            # Bitwise
            "&=": "|=",
            "|=": "&=",
            "^=": "&=",
            # Shift
            "<<=": ">>=",
            ">>=": "<<=",
            ">>>=": "<<=",  # Unsigned right shift
            # Logical (ES2021)
            "&&=": "||=",
            "||=": "&&=",
            "??=": "||=",  # Nullish coalescing assignment
            # Exponentiation
            "**=": "*=",
        }

        # Update expression swaps (++, --)
        update_swaps = {
            "++": "--",
            "--": "++",
        }

        def collect_augmented_assignments(n):
            # Handle augmented assignment expressions (+=, -=, etc.)
            if n.type == "augmented_assignment_expression":
                # Find the operator child
                for child in n.children:
                    op_text = source_bytes[child.start_byte : child.end_byte].decode(
                        "utf-8"
                    )
                    if op_text in aug_assign_swaps and self.flip():
                        changes.append(
                            {"node": child, "new_op": aug_assign_swaps[op_text]}
                        )
                        break

            # Handle update expressions (++, --)
            elif n.type == "update_expression":
                for child in n.children:
                    op_text = source_bytes[child.start_byte : child.end_byte].decode(
                        "utf-8"
                    )
                    if op_text in update_swaps and self.flip():
                        changes.append({"node": child, "new_op": update_swaps[op_text]})
                        break

            for child in n.children:
                collect_augmented_assignments(child)

        collect_augmented_assignments(node)

        if not changes:
            return source_code

        # Work with bytes for modifications
        modified_source = source_bytes
        for change in reversed(changes):
            node = change["node"]
            new_op = change["new_op"]

            modified_source = (
                modified_source[: node.start_byte]
                + new_op.encode("utf-8")
                + modified_source[node.end_byte :]
            )

        return _safe_decode(modified_source, source_code)


class TernaryOperatorSwapModifier(JavaScriptProceduralModifier):
    """Modify ternary operators (condition ? consequent : alternative)"""

    explanation: str = "The ternary operator branches may be swapped or the condition may be incorrect."
    name: str = "func_pm_ternary_swap"
    conditions: list = [CodeProperty.IS_FUNCTION, CodeProperty.HAS_TERNARY]

    def modify(self, code_entity: CodeEntity) -> BugRewrite:
        """Modify ternary operators by swapping branches or negating conditions."""
        parser = Parser(JS_LANGUAGE)
        tree = parser.parse(bytes(code_entity.src_code, "utf8"))

        modified_code = self._modify_ternary(code_entity.src_code, tree.root_node)

        if modified_code == code_entity.src_code:
            return None

        return BugRewrite(
            rewrite=modified_code,
            explanation=self.explanation,
            strategy=self.name,
        )

    def _modify_ternary(self, source_code: str, node) -> str:
        """Find and modify ternary (conditional) expressions."""
        changes = []
        source_bytes = source_code.encode("utf-8")

        def collect_ternary_ops(n):
            # In tree-sitter-javascript, ternary is "ternary_expression"
            if n.type == "ternary_expression" and len(n.children) >= 5:
                # Structure: condition ? consequent : alternative
                # Children: [condition, "?", consequent, ":", alternative]
                condition = None
                consequent = None
                alternative = None

                # Parse children - skip operators
                content_children = [c for c in n.children if c.type not in ["?", ":"]]
                if len(content_children) >= 3:
                    condition = content_children[0]
                    consequent = content_children[1]
                    alternative = content_children[2]

                    if condition and consequent and alternative and self.flip():
                        # Choose modification type randomly
                        mod_type = self.rand.choice(
                            ["swap_branches", "negate_condition"]
                        )
                        changes.append(
                            {
                                "node": n,
                                "condition": condition,
                                "consequent": consequent,
                                "alternative": alternative,
                                "mod_type": mod_type,
                            }
                        )

            for child in n.children:
                collect_ternary_ops(child)

        collect_ternary_ops(node)

        if not changes:
            return source_code

        # Work with bytes for modifications
        modified_source = source_bytes
        for change in reversed(changes):
            node = change["node"]
            condition = change["condition"]
            consequent = change["consequent"]
            alternative = change["alternative"]
            mod_type = change["mod_type"]

            condition_text = _safe_decode(
                source_bytes[condition.start_byte : condition.end_byte]
            )
            consequent_text = _safe_decode(
                source_bytes[consequent.start_byte : consequent.end_byte]
            )
            alternative_text = _safe_decode(
                source_bytes[alternative.start_byte : alternative.end_byte]
            )

            if mod_type == "swap_branches":
                # Swap consequent and alternative: a ? b : c -> a ? c : b
                new_ternary = (
                    f"{condition_text} ? {alternative_text} : {consequent_text}"
                )
            else:  # negate_condition
                # Negate condition: a ? b : c -> !a ? b : c  (but keep branches, so effectively swaps logic)
                # Actually, negating and keeping same branches is same as swapping, so:
                # a ? b : c -> !(a) ? b : c  which equals a ? c : b
                # Let's do: negate condition AND swap branches for different bug pattern
                new_ternary = (
                    f"!({condition_text}) ? {consequent_text} : {alternative_text}"
                )

            modified_source = (
                modified_source[: node.start_byte]
                + new_ternary.encode("utf-8")
                + modified_source[node.end_byte :]
            )

        return _safe_decode(modified_source, source_code)


class FunctionArgumentSwapModifier(JavaScriptProceduralModifier):
    """Swap adjacent arguments in function calls."""

    explanation: str = "The function arguments may be in the wrong order."
    name: str = "func_pm_arg_swap"
    conditions: list = [CodeProperty.IS_FUNCTION, CodeProperty.HAS_FUNCTION_CALL]

    def modify(self, code_entity: CodeEntity) -> BugRewrite:
        """Swap adjacent arguments in function calls."""
        parser = Parser(JS_LANGUAGE)
        tree = parser.parse(bytes(code_entity.src_code, "utf8"))

        modified_code = self._swap_arguments(code_entity.src_code, tree.root_node)

        if modified_code == code_entity.src_code:
            return None

        return BugRewrite(
            rewrite=modified_code,
            explanation=self.explanation,
            strategy=self.name,
        )

    def _swap_arguments(self, source_code: str, node) -> str:
        """Find function calls and swap adjacent arguments."""
        changes = []
        source_bytes = source_code.encode("utf-8")

        def collect_function_calls(n):
            if n.type == "call_expression":
                # Find the arguments node
                args_node = None
                for child in n.children:
                    if child.type == "arguments":
                        args_node = child
                        break

                if args_node:
                    # Get actual arguments (skip parentheses and commas)
                    args = [
                        c for c in args_node.children if c.type not in ["(", ")", ","]
                    ]

                    # Need at least 2 arguments to swap
                    if len(args) >= 2 and self.flip():
                        # Choose which pair to swap
                        swap_idx = self.rand.randint(0, len(args) - 2)
                        changes.append(
                            {
                                "args_node": args_node,
                                "args": args,
                                "swap_idx": swap_idx,
                            }
                        )

            for child in n.children:
                collect_function_calls(child)

        collect_function_calls(node)

        if not changes:
            return source_code

        # Work with bytes for modifications
        modified_source = source_bytes
        for change in reversed(changes):
            args_node = change["args_node"]
            args = change["args"]
            swap_idx = change["swap_idx"]

            # Get the two arguments to swap
            arg1 = args[swap_idx]
            arg2 = args[swap_idx + 1]

            arg1_text = _safe_decode(source_bytes[arg1.start_byte : arg1.end_byte])
            arg2_text = _safe_decode(source_bytes[arg2.start_byte : arg2.end_byte])

            # Reconstruct the arguments list with swapped args
            new_args_parts = []
            for i, arg in enumerate(args):
                if i == swap_idx:
                    new_args_parts.append(arg2_text)
                elif i == swap_idx + 1:
                    new_args_parts.append(arg1_text)
                else:
                    new_args_parts.append(
                        _safe_decode(source_bytes[arg.start_byte : arg.end_byte])
                    )

            new_args = "(" + ", ".join(new_args_parts) + ")"

            modified_source = (
                modified_source[: args_node.start_byte]
                + new_args.encode("utf-8")
                + modified_source[args_node.end_byte :]
            )

        return _safe_decode(modified_source, source_code)



from pathlib import Path
from swebench.harness.constants import (
    APPLY_PATCH_FAIL,
    FAIL_TO_FAIL,
    FAIL_TO_PASS,
    KEY_PREDICTION,
    PASS_TO_FAIL,
    PASS_TO_PASS,
    TESTS_TIMEOUT,
    ResolvedStatus,
    TestStatus,
)
from swebench.harness.grading import get_resolution_status
from swesmith.constants import (
    TEST_OUTPUT_END,
    TEST_OUTPUT_START,
)
from swesmith.profiles import registry


def read_test_output(filename: str):
    content = Path(filename).read_text(errors="replace")
    if APPLY_PATCH_FAIL in content:
        return None, False
    if TESTS_TIMEOUT in content:
        return None, False
    if TEST_OUTPUT_START not in content or TEST_OUTPUT_END not in content:
        return content, False
    start_sep = f"+ : '{TEST_OUTPUT_START}'"
    end_sep = f"+ : '{TEST_OUTPUT_END}'"
    start_idx = content.find(start_sep)
    end_idx = content.find(end_sep)
    if start_idx > end_idx:
        raise ValueError(
            "Invalid test output - Start and end markers are not in correct order"
        )
    return content[start_idx:end_idx][len(start_sep) :], True


def get_valid_report(
    val_pregold_path: str,
    val_postgold_path: str,
    instance: dict,
) -> dict[str, list[str]]:
    """
    Get a report of changes in test pass/fail status between pre-gold and post-gold validation logs

    Args:
        val_pregold_path (str): path to pre-gold validation log
        val_postgold_path (str): path to post-gold validation log
    Returns:
        report (dict): map of type of status change to list of test cases
    """
    rp = registry.get(instance["repo"])

    val_pregold_output, found_pregold = read_test_output(val_pregold_path)
    val_postgold_output, found_postgold = read_test_output(val_postgold_path)
    pregold_sm = rp.log_parser(val_pregold_output) if found_pregold else {}
    postgold_sm = rp.log_parser(val_postgold_output) if found_postgold else {}

    report = {
        FAIL_TO_PASS: [],
        PASS_TO_PASS: [],
        FAIL_TO_FAIL: [],
        PASS_TO_FAIL: [],
    }

    for test_case in postgold_sm:
        if test_case not in pregold_sm:
            continue
        elif (
            pregold_sm[test_case] == TestStatus.PASSED.value
            and postgold_sm[test_case] == TestStatus.PASSED.value
        ):
            report[PASS_TO_PASS].append(test_case)
        elif (
            pregold_sm[test_case] == TestStatus.FAILED.value
            and postgold_sm[test_case] == TestStatus.PASSED.value
        ):
            report[FAIL_TO_PASS].append(test_case)
        elif (
            pregold_sm[test_case] == TestStatus.FAILED.value
            and postgold_sm[test_case] == TestStatus.FAILED.value
        ):
            report[FAIL_TO_FAIL].append(test_case)
        elif (
            pregold_sm[test_case] == TestStatus.PASSED.value
            and postgold_sm[test_case] == TestStatus.FAILED.value
        ):
            report[PASS_TO_FAIL].append(test_case)

    return report


def test_passed(case: str, sm: dict[str, str]) -> bool:
    return case in sm and sm[case] in [
        TestStatus.PASSED.value,
        TestStatus.XFAIL.value,
    ]


def test_failed(case: str, sm: dict[str, str]) -> bool:
    return case not in sm or sm[case] in [
        TestStatus.FAILED.value,
        TestStatus.ERROR.value,
    ]


def get_eval_tests_report(
    eval_status_map: dict[str, str],
    gold_results: dict[str, str],
    calculate_to_fail: bool = False,
) -> dict[str, dict[str, list[str]]]:
    """
    Create a report based on failure/pass change from gold results to eval results.

    Args:
        eval_sm (dict): evaluation status map
        gold_results (dict): gold results
        calculate_to_fail (bool): whether to calculate metrics for "x to fail" tests
    Returns:
        report (dict): report of metrics

    Metric Definitions (Gold Result Pair + Eval Result):
    - Fail-Pass (F2P) + P: Success (Resolution)
    - Pass-Pass (P2P) + P: Success (Maintenance)
    - Fail-Pass (F2P) + F: Failure
    - Pass-Pass (P2P) + F: Failure

    Miscellaneous Definitions
    - Fail-Fail (F2F) + F: Failure Maintenance
    - Pass-Fail (P2F) + F: Not considered
    - Fail-Fail (F2F) + P: Success (Extra Credit)
    - Pass-Fail (P2F) + P: Not considered
    """
    # Calculate resolution metrics
    f2p_success = []
    f2p_failure = []
    for test_case in gold_results[FAIL_TO_PASS]:
        if test_passed(test_case, eval_status_map):
            f2p_success.append(test_case)
        elif test_failed(test_case, eval_status_map):
            f2p_failure.append(test_case)

    # Calculate maintenance metrics
    p2p_success = []
    p2p_failure = []
    for test_case in gold_results[PASS_TO_PASS]:
        if test_passed(test_case, eval_status_map):
            p2p_success.append(test_case)
        elif test_failed(test_case, eval_status_map):
            p2p_failure.append(test_case)

    results = {
        FAIL_TO_PASS: {
            "success": f2p_success,
            "failure": f2p_failure,
        },
        PASS_TO_PASS: {
            "success": p2p_success,
            "failure": p2p_failure,
        },
    }

    f2f_success = []
    f2f_failure = []
    p2f_success = []
    p2f_failure = []
    if calculate_to_fail:
        # Calculate "extra credit" metrics
        for test_case in gold_results[FAIL_TO_FAIL]:
            if test_passed(test_case, eval_status_map):
                f2f_success.append(test_case)
            elif test_failed(test_case, eval_status_map):
                f2f_failure.append(test_case)
        # Calculate not considered metrics
        for test_case in gold_results[PASS_TO_FAIL]:
            if test_passed(test_case, eval_status_map):
                p2f_success.append(test_case)
            elif test_failed(test_case, eval_status_map):
                p2f_failure.append(test_case)

    results.update(
        {
            FAIL_TO_FAIL: {
                "success": f2f_success,
                "failure": f2f_failure,
            },
            PASS_TO_FAIL: {
                "success": p2f_success,
                "failure": p2f_failure,
            },
        }
    )
    return results


def get_eval_report(
    prediction: dict,
    inst: dict,
    test_log_path: str,
    f2p_only: bool = False,
):
    report_map = {
        "patch_exists": False,
        "resolved": False,
    }
    rp = registry.get_from_inst(inst)

    # Check if model patch exists
    if prediction[KEY_PREDICTION] is None:
        return report_map
    report_map["patch_exists"] = True

    # Get evaluation logs
    test_output, found = read_test_output(test_log_path)
    if not found:
        return report_map
    test_status_map = rp.log_parser(test_output)

    if f2p_only:
        # Only examine f2p tests
        f2p_files, _ = rp.get_test_files(inst)
        filter_irrelevant_tests = (
            lambda tests: [
                x for x in tests if any([x.startswith(y) for y in f2p_files])
            ]
            if len(f2p_files) > 0
            else tests
        )
        inst[FAIL_TO_PASS] = filter_irrelevant_tests(inst[FAIL_TO_PASS])
        inst[PASS_TO_PASS] = filter_irrelevant_tests(inst[PASS_TO_PASS])

    # Get evaluation test report
    report = get_eval_tests_report(test_status_map, inst)
    if get_resolution_status(report) == ResolvedStatus.FULL.value:
        report_map["resolved"] = True
    report_map["tests_status"] = report

    return report_map



"""
Purpose: Given a bug patch, retrieve the issue text from the PR that the bug was created from.

python swesmith/issue_gen/get_from_pr.py logs/experiments/*.json
"""

import argparse
import json

from pathlib import Path
from swesmith.constants import LOG_DIR_BUG_GEN
from swesmith.bug_gen.mirror.generate import INSTANCE_REF, MIRROR_PR
from tqdm.auto import tqdm


def transform_to_sweb_inst_id(inst):
    repo = inst["repo"].split("/", 1)[-1].rsplit(".", 1)[0]
    pr_num = inst["instance_id"].rsplit("_", 1)[-1]
    return f"{repo}-{pr_num}"


def get_original_ps_from_pr(instance, log_dir_bug_gen=LOG_DIR_BUG_GEN):
    log_dir_bug_gen = Path(log_dir_bug_gen)
    sweb_inst_id = transform_to_sweb_inst_id(instance)
    pr_num = sweb_inst_id.rsplit("-", 1)[-1]
    metadata_path = (
        log_dir_bug_gen
        / instance["repo"].split("/")[-1]
        / MIRROR_PR
        / sweb_inst_id
        / f"metadata__pr_{pr_num}.json"
    )
    if not metadata_path.exists():
        return ""
    with open(metadata_path, "r") as f:
        metadata = json.load(f)
    if INSTANCE_REF not in metadata:
        return ""
    ps = metadata[INSTANCE_REF]["problem_statement"]
    return ps


def main(dataset_path: str):
    dataset_path = Path(dataset_path)

    # Load bug dataset
    with open(dataset_path, "r") as f:
        dataset = json.load(f)
    print(f"Found {len(dataset)} task instances to generate instructions for")
    kept = []
    for instance in tqdm(dataset):
        ps = get_original_ps_from_pr(instance)
        if len(ps.strip()) > 0:
            instance["problem_statement"] = ps
            kept.append(instance)
    print(
        f"{len(kept)} instances have problem statements ({len(dataset) - len(kept)} missing)"
    )

    if len(kept) > 0:
        # Create .json version of the dataset
        output_path = dataset_path.parent / f"{dataset_path.stem}__ig_orig.json"
        with open(output_path, "w") as f:
            json.dump(kept, f, indent=2)
        print(f"Wrote dataset with original problem statements to {output_path}")
    else:
        print("No instances found with original problem statements.")


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "dataset_path",
        type=str,
        help="Path to the dataset",
    )
    args = parser.parse_args()
    main(**vars(args))



import re

from dataclasses import dataclass, field
from swebench.harness.constants import TestStatus
from swesmith.constants import ENV_NAME
from swesmith.profiles.base import RepoProfile, registry


@dataclass
class CProfile(RepoProfile):
    """
    Profile for C repositories.
    """

    exts: list[str] = field(default_factory=lambda: [".c"])


@dataclass
class Jqb9e19de76(CProfile):
    owner: str = "jqlang"
    repo: str = "jq"
    commit: str = "b9e19de76e6e19d044007ead65d164710dc98877"
    test_cmd: str = "make check"
    eval_sets: set[str] = field(
        default_factory=lambda: {"SWE-bench/SWE-bench_Multilingual"}
    )

    @property
    def dockerfile(self):
        return f"""FROM ubuntu:22.04
ENV DEBIAN_FRONTEND=noninteractive \
    DEBCONF_NONINTERACTIVE_SEEN=true \
    LC_ALL=C.UTF-8 \
    LANG=C.UTF-8
ENV TZ=Etc/UTC
RUN apt-get update \
    && apt-get install -y build-essential autoconf libtool git \
    && apt-get clean \
    && rm -rf /var/lib/apt/lists/*
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN git submodule update --init --recursive
RUN autoreconf -i \
    && ./configure \
    --disable-docs \
    --with-oniguruma=builtin \
    --enable-static \
    --enable-all-static \
    --prefix=/usr/local
RUN make clean
RUN touch src/parser.y src/lexer.l
RUN make -j$(nproc)
"""

    def log_parser(self, log: str) -> dict[str, str]:
        test_status_map = {}
        pattern = r"^\s*(PASS|FAIL):\s(.+)$"
        for line in log.split("\n"):
            match = re.match(pattern, line.strip())
            if match:
                status, test_name = match.groups()
                if status == "PASS":
                    test_status_map[test_name] = TestStatus.PASSED.value
                elif status == "FAIL":
                    test_status_map[test_name] = TestStatus.FAILED.value
        return test_status_map


@dataclass
class Valkeyfc7c04e4(CProfile):
    owner: str = "valkey-io"
    repo: str = "valkey"
    commit: str = "fc7c04e4f8ba86dfbac1ec059db457fb44ed0a2d"
    test_cmd: str = "TERM=dumb ./runtest --durable"
    eval_sets: set[str] = field(
        default_factory=lambda: {"SWE-bench/SWE-bench_Multilingual"}
    )

    @property
    def dockerfile(self):
        return f"""FROM ubuntu:22.04
ARG DEBIAN_FRONTEND=noninteractive
ENV TZ=Etc/UTC
RUN sed -i 's/^# deb-src/deb-src/' /etc/apt/sources.list
RUN apt update && \
    apt install -y pkg-config wget git build-essential libtool automake autoconf tcl bison flex cmake python3 python3-pip python3-venv python-is-python3 && \
    rm -rf /var/lib/apt/lists/*
RUN adduser --disabled-password --gecos 'dog' nonroot
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN cd deps/jemalloc && ./autogen.sh
RUN make distclean
RUN make
"""

    def log_parser(self, log: str) -> dict[str, str]:
        test_status_map = {}
        pattern = r"^\[(ok|err|skip|ignore)\]:\s(.+?)(?:\s\((\d+\s*m?s)\))?$"
        for line in log.split("\n"):
            match = re.match(pattern, line.strip())
            if match:
                status, test_name, _duration = match.groups()
                if status == "ok":
                    test_status_map[test_name] = TestStatus.PASSED.value
                elif status == "err":
                    # Strip out file path information from failed test names
                    test_name = re.sub(r"\s+in\s+\S+$", "", test_name)
                    test_status_map[test_name] = TestStatus.FAILED.value
                elif status == "skip" or status == "ignore":
                    test_status_map[test_name] = TestStatus.SKIPPED.value
        return test_status_map


# Register all C profiles with the global registry
for name, obj in list(globals().items()):
    if (
        isinstance(obj, type)
        and issubclass(obj, CProfile)
        and obj.__name__ != "CProfile"
    ):
        registry.register_profile(obj)



#!/usr/bin/env python3

"""Given the `<owner/name>` of a GitHub repo, this script writes the raw information for all the repo's PRs to a single `.jsonl` file."""

from __future__ import annotations

import argparse
import json
import logging
import os

from datetime import datetime
from fastcore.xtras import obj2dict
from swesmith.bug_gen.mirror.collect.utils import Repo
from typing import Optional

logging.basicConfig(
    level=logging.INFO, format="%(asctime)s - %(name)s - %(levelname)s - %(message)s"
)
logger = logging.getLogger(__name__)


def log_all_pulls(
    repo: Repo,
    output: str,
    max_pulls: int = None,
    cutoff_date: str = None,
) -> None:
    """
    Iterate over all pull requests in a repository and log them to a file

    Args:
        repo (Repo): repository object
        output (str): output file name
    """
    cutoff_date = (
        datetime.strptime(cutoff_date, "%Y%m%d").strftime("%Y-%m-%dT%H:%M:%SZ")
        if cutoff_date is not None
        else None
    )

    with open(output, "w") as file:
        for i_pull, pull in enumerate(repo.get_all_pulls()):
            setattr(pull, "resolved_issues", repo.extract_resolved_issues(pull))
            print(json.dumps(obj2dict(pull)), end="\n", flush=True, file=file)
            if max_pulls is not None and i_pull >= max_pulls:
                break
            if cutoff_date is not None and pull.created_at < cutoff_date:
                break


def log_single_pull(
    repo: Repo,
    pull_number: int,
    output: str,
) -> None:
    """
    Get a single pull request from a repository and log it to a file

    Args:
        repo (Repo): repository object
        pull_number (int): pull request number
        output (str): output file name
    """
    logger.info(f"Fetching PR #{pull_number} from {repo.owner}/{repo.name}")

    # Get the pull request using the GitHub API
    pull = repo.call_api(
        repo.api.pulls.get, owner=repo.owner, repo=repo.name, pull_number=pull_number
    )

    if pull is None:
        logger.error(f"PR #{pull_number} not found in {repo.owner}/{repo.name}")
        return

    # Extract resolved issues
    setattr(pull, "resolved_issues", repo.extract_resolved_issues(pull))

    # Log the pull request to a file
    with open(output, "w") as file:
        print(json.dumps(obj2dict(pull)), end="\n", flush=True, file=file)

    logger.info(f"PR #{pull_number} saved to {output}")
    logger.info(f"Resolved issues: {pull.resolved_issues}")


def main(
    repo_name: str,
    output: str,
    token: Optional[str] = None,
    max_pulls: int = None,
    cutoff_date: str = None,
    pull_number: int = None,
):
    """
    Logic for logging all pull requests in a repository

    Args:
        repo_name (str): name of the repository
        output (str): output file name
        token (str, optional): GitHub token
        max_pulls (int, optional): maximum number of pulls to log
        cutoff_date (str, optional): cutoff date for PRs to consider
        pull_number (int, optional): specific pull request number to log
    """
    if token is None:
        token = os.environ.get("GITHUB_TOKEN")
    owner, repo = repo_name.split("/")
    repo = Repo(owner, repo, token=token)

    if pull_number is not None:
        log_single_pull(repo, pull_number, output)
    else:
        log_all_pulls(repo, output, max_pulls=max_pulls, cutoff_date=cutoff_date)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument("repo_name", type=str, help="Name of the repository")
    parser.add_argument("output", type=str, help="Output file name")
    parser.add_argument("--token", type=str, help="GitHub token")
    parser.add_argument(
        "--max_pulls", type=int, help="Maximum number of pulls to log", default=None
    )
    parser.add_argument(
        "--cutoff_date",
        type=str,
        help="Cutoff date for PRs to consider in format YYYYMMDD",
        default=None,
    )
    parser.add_argument(
        "--pull_number",
        type=int,
        help="Specific pull request number to log",
        default=None,
    )
    args = parser.parse_args()
    main(**vars(args))



import re
import tree_sitter_cpp as tscpp

from swesmith.constants import TODO_REWRITE, CodeEntity
from tree_sitter import Language, Parser, Query, QueryCursor
from swesmith.bug_gen.adapters.utils import build_entity

CPP_LANGUAGE = Language(tscpp.language())


class CPlusPlusEntity(CodeEntity):
    @property
    def name(self) -> str:
        func_query = Query(
            CPP_LANGUAGE,
            """
            [
                (function_definition (function_declarator declarator: (identifier) @name))
                (function_definition (function_declarator declarator: (destructor_name (identifier) @name)))
            ]
            """,
        )
        matches = QueryCursor(func_query).matches(self.node)
        if matches:
            name_node = matches[0][1]["name"][0]
            func_name = name_node.text.decode("utf-8")
            if name_node.parent.type == "destructor_name":
                return f"{func_name} Destructor"
            return func_name
        return ""

    @property
    def signature(self) -> str:
        body_query = Query(
            CPP_LANGUAGE, "(function_definition body: (compound_statement) @body)"
        )
        matches = QueryCursor(body_query).matches(self.node)
        if matches:
            body_node = matches[0][1]["body"][0]
            body_start_byte = body_node.start_byte - self.node.start_byte
            signature = self.node.text[:body_start_byte].strip().decode("utf-8")
            signature = re.sub(r"\(\s+", "(", signature).strip()
            signature = re.sub(r"\s+\)", ")", signature).strip()
            signature = re.sub(r"\s+", " ", signature).strip()
            return signature
        return ""

    @property
    def stub(self) -> str:
        return f"{self.signature} {{\n\t// {TODO_REWRITE}\n}}"


def get_entities_from_file_cpp(
    entities: list[CPlusPlusEntity],
    file_path: str,
    max_entities: int = -1,
) -> None:
    """
    Parse a .cpp file and return up to max_entities top-level funcs.
    If max_entities < 0, collects them all.
    """
    parser = Parser(CPP_LANGUAGE)

    file_content = open(file_path, "r", encoding="utf8").read()
    tree = parser.parse(bytes(file_content, "utf8"))
    root = tree.root_node
    lines = file_content.splitlines()

    def walk(node) -> None:
        # stop if we've hit the limit
        if 0 <= max_entities == len(entities):
            return

        # not checking for error nodes here because tree-sitter-cpp frequently
        # generates them parsing valid pre-processor directives

        if node.type == "function_definition":
            entities.append(build_entity(node, lines, file_path, CPlusPlusEntity))
            if 0 <= max_entities == len(entities):
                return

        for child in node.children:
            walk(child)

    walk(root)



"""
Base classes and utilities for procedural bug generation across different languages.

This module provides the foundational infrastructure for language-specific procedural
modification techniques. Language-specific implementations should be placed in their
respective subdirectories (e.g., python/, javascript/, java/).
"""

# For backward compatibility, expose Python-specific classes
from swesmith.bug_gen.procedural.golang import MODIFIERS_GOLANG
from swesmith.bug_gen.procedural.python import MODIFIERS_PYTHON
from swesmith.bug_gen.procedural.javascript import MODIFIERS_JAVASCRIPT
from swesmith.bug_gen.procedural.rust import MODIFIERS_RUST

MAP_EXT_TO_MODIFIERS = {
    ".go": MODIFIERS_GOLANG,
    ".py": MODIFIERS_PYTHON,
    ".js": MODIFIERS_JAVASCRIPT,
    ".rs": MODIFIERS_RUST,
}



"""
Purpose: Given a task instance, attached a fixed problem statement to the issue text.

python swesmith/issue_gen/get_fixed.py logs/experiments/*.json
"""

import argparse
import json
import random
from typing import Set

from pathlib import Path
from swebench.harness.constants import FAIL_TO_PASS, KEY_INSTANCE_ID
from swesmith.bug_gen.procedural.generate import (
    PM_TECHNIQUES_CLASSES,
    PM_TECHNIQUES_FUNCS,
)
from tqdm.auto import tqdm
from unidiff import PatchSet

BUG_TYPE_TO_PROMPT = {
    x.name: x.explanation for x in PM_TECHNIQUES_CLASSES + PM_TECHNIQUES_FUNCS
}

# MARK: Basic says-nothing prompt
PROMPT_BASIC = (
    """There is a bug in this codebase. Please look into it and resolve the issue."""
)

# MARK: Prompts that mention file names
PROMPT_FILES = """There are bug(s) in this codebase, likely located in the following file(s):
{gold_files}

Please look into them and fix any bugs that you find."""

# MARK: Prompts that mention file + function names
PROMPT_FILES_FUNCS = """There are bug(s) in this codebase, likely located in the following file(s).
{gold_files}

I think these function(s) are relevant to the bug:
{gold_funcs}

Please look into them and fix any bugs that you find."""

# MARK: Prompts that mention test cases
PROMPT_TESTS_BASIC = (
    """Several tests in the codebase are breaking. Please find the bugs and fix them."""
)
PROMPT_TESTS_F2P = """Several tests in the codebase are breaking.

The tests that are failing are:
{f2p_list}

Please fix the codebase such that the tests pass."""

# MARK: Prompts that mention the type of bug
PROMPT_BUG_TYPE_BASIC = """There is a bug in this codebase. {bug_type}Please look into it and resolve the issue."""
PROMPT_BUG_TYPE_FILES = """There is a bug in this codebase. {bug_type}It seems to be related to the following files:" \
{gold_files}
Please look into these files and resolve the issue."""
PROMPT_BUG_TYPE_FILES_TESTS = """There is a bug in this codebase. {bug_type}It seems to be related to the following files:
{gold_files}

Please look into these files and resolve the issue. I believe a test case is also failing because of this bug:
{f2p_single}"""
PROMPT_BUG_TYPE_FILES_FUNCS_TESTS = """There is a bug in this codebase. {bug_type}It seems to be related to the following files:
{gold_files}

I think these function(s) are relevant to the bug:
{gold_funcs}

Please look into this and resolve the issue. I believe a test case is also failing because of this bug:
{f2p_single}"""

PROMPT_POOL = [
    (PROMPT_BASIC, 0.05),
    (PROMPT_FILES, 0.1),
    (PROMPT_FILES_FUNCS, 0.15),
    (PROMPT_TESTS_BASIC, 0.1),
    (PROMPT_TESTS_F2P, 0.1),
    (PROMPT_BUG_TYPE_BASIC, 0.05),
    (PROMPT_BUG_TYPE_FILES, 0.15),
    (PROMPT_BUG_TYPE_FILES_TESTS, 0.15),
    (PROMPT_BUG_TYPE_FILES_FUNCS_TESTS, 0.15),
]

random.seed(24)


def print_list(x):
    return "- " + "\n- ".join(x)


def get_bug_exp(instance) -> str:
    inst_id = instance[KEY_INSTANCE_ID]
    for bug_type, prompt in BUG_TYPE_TO_PROMPT.items():
        if bug_type in inst_id:
            return prompt
    return ""


def get_changed_functions(patch_text) -> Set[str]:
    patch = PatchSet(patch_text.splitlines())
    changed_funcs = set()

    for file in patch:
        for hunk in file:
            for line in hunk:
                if line.is_added or line.is_removed:
                    # Extract function context
                    function_name = hunk.section_header
                    if function_name:
                        changed_funcs.add(function_name.strip())

    return changed_funcs


def main(dataset_path: str | Path) -> None:
    dataset_path = Path(dataset_path)
    dataset = []
    if dataset_path.name.endswith(".json"):
        with open(dataset_path, "r") as f:
            dataset = json.load(f)
    elif dataset_path.name.endswith(".jsonl"):
        with open(dataset_path, "r") as f:
            dataset = [json.loads(x) for x in f]
    else:
        raise ValueError(
            f"Unsupported file format (must be .json, .jsonl): {dataset_path}"
        )
    dataset_path = Path(dataset_path)
    print(f"Found {len(dataset)} task instances to generate instructions for")

    prompt_pool = [x[0] for x in PROMPT_POOL]
    prompt_weights = [x[1] for x in PROMPT_POOL]
    for instance in tqdm(dataset):
        instance["bug_type"] = get_bug_exp(instance)
        instance["f2p_single"] = random.choice(instance[FAIL_TO_PASS])
        instance["f2p_list"] = print_list(instance[FAIL_TO_PASS])
        instance["gold_files"] = print_list(
            [x.path for x in PatchSet(instance["patch"])]
        )
        instance["gold_funcs"] = print_list(get_changed_functions(instance["patch"]))

        prompt = random.choices(prompt_pool, weights=prompt_weights, k=1)[0]
        instance["problem_statement"] = prompt.format(**instance)
    out_path = dataset_path.parent / f"{dataset_path.stem}__ig_static.json"
    with open(out_path, "w") as f:
        json.dump(dataset, f, indent=2)
        print(f"Wrote dataset with static instructions to {out_path}")


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("dataset_path", type=str, help="Path to the dataset file")
    args = parser.parse_args()
    main(**vars(args))



import os
import re
import shutil

from dataclasses import dataclass, field
from swebench.harness.constants import (
    FAIL_TO_PASS,
    PASS_TO_PASS,
    KEY_INSTANCE_ID,
    TestStatus,
)
from swesmith.constants import ENV_NAME
from swesmith.profiles.base import RepoProfile, registry


@dataclass
class GoProfile(RepoProfile):
    """
    Profile for Golang repositories.

    This class provides Golang-specific defaults and functionality for
    repository profiles.
    """

    exts: list[str] = field(default_factory=lambda: [".go"])
    test_cmd: str = "go test -v ./..."
    _test_name_to_files_cache: dict[str, set[str]] = field(
        default=None, init=False, repr=False
    )

    @property
    def dockerfile(self):
        return f"""FROM golang:1.24
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN go mod tidy
RUN go test -v -count=1 ./... || true
"""

    def _build_test_name_to_files_map(self) -> dict[str, set[str]]:
        """Build a mapping from test names to the files that contain them."""
        dest, cloned = self.clone()
        test_name_to_files = {}

        # Scan all test files once
        for dirpath, _, filenames in os.walk(dest):
            for fname in filenames:
                if not fname.endswith("_test.go"):
                    continue

                full_path = os.path.join(dirpath, fname)
                # Convert to relative path from repository root
                relative_path = os.path.relpath(full_path, dest)

                try:
                    with open(full_path, "r", encoding="utf-8") as f:
                        for line in f:
                            # Look for function definitions that are tests
                            match = re.match(r"^\s*func\s+(\w+)\b", line.strip())
                            if match:
                                test_name = match.group(1)
                                if test_name not in test_name_to_files:
                                    test_name_to_files[test_name] = set()
                                test_name_to_files[test_name].add(relative_path)
                except (OSError, UnicodeDecodeError):
                    # skip files we can't read
                    continue

        if cloned:
            shutil.rmtree(dest)
        return test_name_to_files

    def get_test_files(self, instance: dict) -> tuple[list[str], list[str]]:
        assert FAIL_TO_PASS in instance and PASS_TO_PASS in instance, (
            f"Instance {instance[KEY_INSTANCE_ID]} missing required keys {FAIL_TO_PASS} or {PASS_TO_PASS}"
        )

        # Lazy load the cache if needed
        if self._test_name_to_files_cache is None:
            with self._lock:  # Only one process enters this block at a time
                if self._test_name_to_files_cache is None:  # Double-check pattern
                    self._test_name_to_files_cache = (
                        self._build_test_name_to_files_map()
                    )

        # Look up each test name in the cache
        f2p_files = set()
        for test_name in instance[FAIL_TO_PASS]:
            if test_name in self._test_name_to_files_cache:
                f2p_files.update(self._test_name_to_files_cache[test_name])

        p2p_files = set()
        for test_name in instance[PASS_TO_PASS]:
            if test_name in self._test_name_to_files_cache:
                p2p_files.update(self._test_name_to_files_cache[test_name])

        return list(f2p_files), list(p2p_files)

    def log_parser(self, log: str) -> dict[str, str]:
        """Parser for test logs generated with 'go test'"""
        test_status_map = {}

        pattern_status_map = [
            (re.compile(r"--- PASS: (\S+)"), TestStatus.PASSED.value),
            (re.compile(r"--- FAIL: (\S+)"), TestStatus.FAILED.value),
            (re.compile(r"FAIL:?\s?(.+?)\s"), TestStatus.FAILED.value),
            (re.compile(r"--- SKIP: (\S+)"), TestStatus.SKIPPED.value),
        ]
        for line in log.split("\n"):
            for pattern, status in pattern_status_map:
                match = pattern.match(line.strip())
                if match:
                    test_name = match.group(1)
                    test_status_map[test_name] = status
                    break

        return test_status_map


@dataclass
class Gin3c12d2a8(GoProfile):
    owner: str = "gin-gonic"
    repo: str = "gin"
    commit: str = "3c12d2a80e40930632fc4a4a4e1a45140f33fb12"
    eval_sets: set[str] = field(
        default_factory=lambda: {"SWE-bench/SWE-bench_Multilingual"}
    )


@dataclass
class Fzf976001e4(GoProfile):
    owner: str = "junegunn"
    repo: str = "fzf"
    commit: str = "976001e47459973b5e72565f3047cc9d9e20241d"


@dataclass
class Caddy77dd12cc(GoProfile):
    owner: str = "caddyserver"
    repo: str = "caddy"
    commit: str = "77dd12cc785990c5c5da947b4e883029ab8bd552"
    eval_sets: set[str] = field(
        default_factory=lambda: {"SWE-bench/SWE-bench_Multilingual"}
    )


@dataclass
class Frp61330d4d(GoProfile):
    owner: str = "fatedier"
    repo: str = "frp"
    commit: str = "61330d4d794180c38d1f8ff7e9024b7f0f69d717"


@dataclass
class Gorm1e8baf54(GoProfile):
    owner: str = "go-gorm"
    repo: str = "gorm"
    commit: str = "1e8baf545953dd58e2f301f4dfef5febbc12da0f"


@dataclass
class Echo98ca08e7(GoProfile):
    owner: str = "labstack"
    repo: str = "echo"
    commit: str = "98ca08e7dd64075b858e758d6693bf9799340756"


@dataclass
class Natsserver2ee2e24c(GoProfile):
    owner: str = "nats-io"
    repo: str = "nats-server"
    commit: str = "2ee2e24cb10924adb699ecb68b89e8ce2523ea75"
    timeout: int = 120


@dataclass
class Addressec203a4f(GoProfile):
    owner: str = "bojanz"
    repo: str = "address"
    commit: str = "ec203a4f7f569c03a0f83e2e749b63947481fe4c"


@dataclass
class Goatcounter854b1dd2(GoProfile):
    owner: str = "arp242"
    repo: str = "goatcounter"
    commit: str = "854b1dd2408ca95645ad03ea3fd01ccfe267261a"


@dataclass
class Gotests16a93f6e(GoProfile):
    owner: str = "cweill"
    repo: str = "gotests"
    commit: str = "16a93f6eb6519118b1d282e2f233596a98dd7e96"


@dataclass
class Aferof5375068(GoProfile):
    owner: str = "spf13"
    repo: str = "afero"
    commit: str = "f5375068505ede77db8f13bfb1069011fab77063"


@dataclass
class Color5a495618(GoProfile):
    owner: str = "gookit"
    repo: str = "color"
    commit: str = "5a4956180d841b68a25a68cb632aea0128845a5a"


@dataclass
class Goprompt82a91227(GoProfile):
    owner: str = "c-bata"
    repo: str = "go-prompt"
    commit: str = "82a912274504477990ecf7c852eebb7c85291772"


@dataclass
class Accounting(GoProfile):
    owner: str = "leekchan"
    repo: str = "accounting"
    commit: str = "2e09117338f81558182056c197506abceadc83e0"


@dataclass
class Mpb(GoProfile):
    owner: str = "vbauerster"
    repo: str = "mpb"
    commit: str = "d30b560650ec806c82029422335904404814e220"


@dataclass
class Bubbletea(GoProfile):
    owner: str = "charmbracelet"
    repo: str = "bubbletea"
    commit: str = "ca9473b2d93dc3abce4f8b634e11a4b351517a84"


@dataclass
class Fx(GoProfile):
    owner: str = "antonmedv"
    repo: str = "fx"
    commit: str = "1ab8a99b7cfd5bb4242677a5215e000c69b8b9e0"


@dataclass
class UIProgress(GoProfile):
    owner: str = "gosuri"
    repo: str = "uiprogress"
    commit: str = "484b9f69ea000422e1873db136dbb80e30b5de3c"


@dataclass
class Cobra(GoProfile):
    owner: str = "spf13"
    repo: str = "cobra"
    commit: str = "6dec1ae26659a130bdb4c985768d1853b0e1bc06"


@dataclass
class GoFlags(GoProfile):
    owner: str = "jessevdk"
    repo: str = "go-flags"
    commit: str = "8eae68f0a7870eec41bc8061c2194040048cdf59"


@dataclass
class PFlag(GoProfile):
    owner: str = "spf13"
    repo: str = "pflag"
    commit: str = "1c62fb2813da5f1d1b893a49180a41b3f6be3262"


@dataclass
class Liner(GoProfile):
    owner: str = "peterh"
    repo: str = "liner"
    commit: str = "58a158787cd552b11ce4a45f589a5452072c1fc0"


@dataclass
class Env(GoProfile):
    owner: str = "caarlos0"
    repo: str = "env"
    commit: str = "56a09d295d9321b1f3b537fd23df1527011cd83d"


@dataclass
class Godotenv(GoProfile):
    owner: str = "joho"
    repo: str = "godotenv"
    commit: str = "3a7a19020151b45a29896c9142723efe5b11a061"


@dataclass
class Hjsongo(GoProfile):
    owner: str = "hjson"
    repo: str = "hjson-go"
    commit: str = "f3219653412abdb7bf061c55f58bde481db46051"


@dataclass
class Sonic(GoProfile):
    owner: str = "bytedance"
    repo: str = "sonic"
    commit: str = "de4f017fca6448580003b6cc661bed8fded68d1d"


@dataclass
class Muffet(GoProfile):
    owner: str = "raviqqe"
    repo: str = "muffet"
    commit: str = "430e693772b88a413ff23214e026daff3f05f82a"


@dataclass
class Omniparser(GoProfile):
    owner: str = "jf-tech"
    repo: str = "omniparser"
    commit: str = "d4371ab77afacd626b21d925e0e5b7989298e847"


@dataclass
class Roaring(GoProfile):
    owner: str = "RoaringBitmap"
    repo: str = "roaring"
    commit: str = "09c46a0a47d21ebbe4bedb01bbcf0ba96f22a46d"


@dataclass
class Bitset(GoProfile):
    owner: str = "bits-and-blooms"
    repo: str = "bitset"
    commit: str = "167865a24c4956c76a987f0a7612f9ac04b93a82"


@dataclass
class BoomFilters(GoProfile):
    owner: str = "tylertreat"
    repo: str = "BoomFilters"
    commit: str = "db6545748bc4726eb9410c6763c7e4035d6ccba3"

    @property
    def dockerfile(self):
        return f"""FROM golang:1.24
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN go mod init github.com/tylertreat/BoomFilters
RUN go mod tidy
"""


@dataclass
class Ini(GoProfile):
    owner: str = "go-ini"
    repo: str = "ini"
    commit: str = "b2f570e5b5b844226bbefe6fb521d891f529a951"

    @property
    def dockerfile(self):
        return f"""FROM golang:1.24
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN go mod init github.com/go-ini/ini
RUN go mod tidy
"""


@dataclass
class GoDatastructures(GoProfile):
    owner: str = "Workiva"
    repo: str = "go-datastructures"
    commit: str = "18d77378f834b72b39509b12f70f3f9915c56884"


@dataclass
class Gods(GoProfile):
    owner: str = "emirpasic"
    repo: str = "gods"
    commit: str = "1d83d5ae39fbb0de45a60365791ff1c8b9bae953"


@dataclass
class Gota(GoProfile):
    owner: str = "go-gota"
    repo: str = "gota"
    commit: str = "f70540952827cfc8abfa1257391fd33284300b24"


@dataclass
class GolangSet(GoProfile):
    owner: str = "deckarep"
    repo: str = "golang-set"
    commit: str = "9480c3eb4dae7f17ca7edac65e4b48690c199993"


@dataclass
class Bleve(GoProfile):
    owner: str = "blevesearch"
    repo: str = "bleve"
    commit: str = "f2876b5e34763ac2d28a75a87dce5f2ff4a64d42"
    timeout: int = 120
    timeout_ref: int = 120


@dataclass
class GoAdaptiveRadixTree(GoProfile):
    owner: str = "plar"
    repo: str = "go-adaptive-radix-tree"
    commit: str = "63c2eff3ccd16d8ae93d963e4b9b33d8f537f82a"


@dataclass
class Trie(GoProfile):
    owner: str = "derekparker"
    repo: str = "trie"
    commit: str = "4095f8e392f77af6b669912d51d17233582a1ba9"


@dataclass
class Bigcache(GoProfile):
    owner: str = "allegro"
    repo: str = "bigcache"
    commit: str = "5aa251c4cc3d607bbb48b825ef583ad1fafa1845"


@dataclass
class Cache2go(GoProfile):
    owner: str = "muesli"
    repo: str = "cache2go"
    commit: str = "518229cd8021d8568e4c6c13743bb050dc1f3a05"


@dataclass
class Fastcache(GoProfile):
    owner: str = "VictoriaMetrics"
    repo: str = "fastcache"
    commit: str = "b7ccf30b0eb69939f4031063a57ec4124f964b00"


@dataclass
class Gcache(GoProfile):
    owner: str = "bluele"
    repo: str = "gcache"
    commit: str = "d8b7e051c564c174fea6ef60d180abf601099015"


@dataclass
class Groupcache(GoProfile):
    owner: str = "golang"
    repo: str = "groupcache"
    commit: str = "2c02b8208cf8c02a3e358cb1d9b60950647543fc"


@dataclass
class Otter(GoProfile):
    owner: str = "maypok86"
    repo: str = "otter"
    commit: str = "20ae57f9b2e4400638be8da5183163d410f0186b"


@dataclass
class Ristretto(GoProfile):
    owner: str = "hypermodeinc"
    repo: str = "ristretto"
    commit: str = "da5701167d70aac45473f5ea98099b118505eef5"


@dataclass
class Sturdyc(GoProfile):
    owner: str = "viccon"
    repo: str = "sturdyc"
    commit: str = "97fc006bbf4a7f1f09922fa77a9444e5ce3a20ad"


@dataclass
class Ttlcache(GoProfile):
    owner: str = "jellydator"
    repo: str = "ttlcache"
    commit: str = "7145e12e34f243c69a0f7b5f6b86a832ad8b4fc8"


@dataclass
class Ledisdb(GoProfile):
    owner: str = "ledisdb"
    repo: str = "ledisdb"
    commit: str = "d35789ec47e667726160e227e7c05e09627a6d6c"


@dataclass
class Buntdb(GoProfile):
    owner: str = "tidwall"
    repo: str = "buntdb"
    commit: str = "3daff4e1233584685027938bde39971cc239f2b2"


@dataclass
class Diskv(GoProfile):
    owner: str = "peterbourgon"
    repo: str = "diskv"
    commit: str = "2566386005f64f58f34e1ff32907800a64537e6a"


@dataclass
class Eliasdb(GoProfile):
    owner: str = "krotik"
    repo: str = "eliasdb"
    commit: str = "88a1da66df9527aa97e8781dfc91cb9feb08125c"


@dataclass
class Godis(GoProfile):
    owner: str = "HDT3213"
    repo: str = "godis"
    commit: str = "8a81b9112aa50d5ae07584291ddb9f80122f0246"


@dataclass
class Moss(GoProfile):
    owner: str = "couchbase"
    repo: str = "moss"
    commit: str = "bf10bab20a24b43c15d23b530fc848e7bb580cad"


@dataclass
class Pogreb(GoProfile):
    owner: str = "akrylysov"
    repo: str = "pogreb"
    commit: str = "76e9512dfd3d100f0032dfa30e77a447b3cbe65c"


@dataclass
class Redka(GoProfile):
    owner: str = "nalgeon"
    repo: str = "redka"
    commit: str = "7c532df931237186942d480e00129ae9436da7ad"


@dataclass
class Rosedb(GoProfile):
    owner: str = "rosedblabs"
    repo: str = "rosedb"
    commit: str = "4af513fe955f755f7af391e6466b09f50ae8cd7f"


@dataclass
class Atlas(GoProfile):
    owner: str = "ariga"
    repo: str = "atlas"
    commit: str = "1afaaba2acfdae2a0c940e784a5465e9be00155d"


@dataclass
class Avro(GoProfile):
    owner: str = "hamba"
    repo: str = "avro"
    commit: str = "ec06b38c0b47ba397a439479d9f43dc92d547b5"


@dataclass
class Skeema(GoProfile):
    owner: str = "skeema"
    repo: str = "skeema"
    commit: str = "defb0097f48c8dfd2d239c6fff4259000fcfee59"


@dataclass
class Chproxy(GoProfile):
    owner: str = "ContentSquare"
    repo: str = "chproxy"
    commit: str = "a9364c8b7923adfb1bf02d28e2298bf46eec5559"


@dataclass
class ClickhouseBulk(GoProfile):
    owner: str = "nikepan"
    repo: str = "clickhouse-bulk"
    commit: str = "cdc261cb029f4d493fa825a6edffe3f2f1b81f1e"


@dataclass
class Prest(GoProfile):
    owner: str = "prest"
    repo: str = "prest"
    commit: str = "c54ddd30b1ed3ebe24bb1dc3db696d107e5d40c4"


@dataclass
class Rdb(GoProfile):
    owner: str = "HDT3213"
    repo: str = "rdb"
    commit: str = "087190b9f7c7cee3c47192f6cdc9197bf6f30265"


@dataclass
class Goqu(GoProfile):
    owner: str = "doug-martin"
    repo: str = "goqu"
    commit: str = "21b6e6d1cb1befe839044764d8ad6b1c6f0b5ef4"


@dataclass
class Squirrel(GoProfile):
    owner: str = "Masterminds"
    repo: str = "squirrel"
    commit: str = "1ded5784535dcffa4e175d4efbd1ca2706927758"


@dataclass
class Sqlingo(GoProfile):
    owner: str = "lqs"
    repo: str = "sqlingo"
    commit: str = "ed36ef030f789fb664e8d22d63bc03eceb45343d"

    @property
    def dockerfile(self):
        return f"""FROM golang:1.24
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN go mod init github.com/lqs/sqlingo
RUN go mod tidy
"""


@dataclass
class Dotsql(GoProfile):
    owner: str = "qustavo"
    repo: str = "dotsql"
    commit: str = "5d06b8903af8416d86b205c175b22ee903d869c8"


@dataclass
class GoMssqldb(GoProfile):
    owner: str = "denisenkom"
    repo: str = "go-mssqldb"
    commit: str = "103f0369fa02aac21aae282e4f7f81c903aba6be"


@dataclass
class Mysql(GoProfile):
    owner: str = "go-sql-driver"
    repo: str = "mysql"
    commit: str = "76c00e35a8d48f8f70f0e7dffe584692bd3fa612"


@dataclass
class GoSqlite3(GoProfile):
    owner: str = "mattn"
    repo: str = "go-sqlite3"
    commit: str = "f76bae4b0044cbba8fb2c72b8e4559e8fbcffd86"


@dataclass
class Godror(GoProfile):
    owner: str = "godror"
    repo: str = "godror"
    commit: str = "cc3b65ef71b255472470aad349098e6663cba6cf"


@dataclass
class Ksql(GoProfile):
    owner: str = "VinGarcia"
    repo: str = "ksql"
    commit: str = "dadb4199eea95cfc4499f9bf4001feccbea86afd"


@dataclass
class Richgo(GoProfile):
    owner: str = "kyoh86"
    repo: str = "richgo"
    commit: str = "98af5f3a762dabdd7f3c30a122a7950fc3cdb4f1"


@dataclass
class Gotests(GoProfile):
    owner: str = "cweill"
    repo: str = "gotests"
    commit: str = "16a93f6eb6519118b1d282e2f233596a98dd7e96"


@dataclass
class GoImportsReviser(GoProfile):
    owner: str = "incu6us"
    repo: str = "goimports-reviser"
    commit: str = "fb560c58db94476809ad5d99d4171dc0db4000d2"


@dataclass
class Wrapcheck(GoProfile):
    owner: str = "tomarrell"
    repo: str = "wrapcheck"
    commit: str = "486d5bbebfef0d94d5ff15b57e01821f6407bb52"


@dataclass
class Todocheck(GoProfile):
    owner: str = "presmihaylov"
    repo: str = "todocheck"
    commit: str = "f0fae9b573374fc0df2ff7f07a7f4693602ae846"


@dataclass
class Revive(GoProfile):
    owner: str = "mgechev"
    repo: str = "revive"
    commit: str = "03e81029a89342ec7107a3655241f479065e208d"


@dataclass
class Errcheck(GoProfile):
    owner: str = "kisielk"
    repo: str = "errcheck"
    commit: str = "dacab891ef4a1c38ecf6c4d94fd66746bb1247d5"


@dataclass
class Dupl(GoProfile):
    owner: str = "mibk"
    repo: str = "dupl"
    commit: str = "1bf052b6e6431cb666549323351baf3b2aa741e4"


@dataclass
class GoCritic(GoProfile):
    owner: str = "go-critic"
    repo: str = "go-critic"
    commit: str = "db2ec6f4d1f42bbe7fe2cd47f311243bbd1b3398"


@dataclass
class GoModOutdated(GoProfile):
    owner: str = "psampaz"
    repo: str = "go-mod-outdated"
    commit: str = "bb79367d102a05221196613dde574f1a0b81b556"


@dataclass
class Xpath(GoProfile):
    owner: str = "antchfx"
    repo: str = "xpath"
    commit: str = "8d50c252d867285812177ffd3ff0924104ffb1eb"


@dataclass
class Bone(GoProfile):
    owner: str = "go-zoo"
    repo: str = "bone"
    commit: str = "31c3a0bb520c6d7a63dbb942459a3067787a975e"


@dataclass
class Chi(GoProfile):
    owner: str = "go-chi"
    repo: str = "chi"
    commit: str = "23c395f8524a30334126ca16fb4d37b88745b9b9"


@dataclass
class Httprouter(GoProfile):
    owner: str = "julienschmidt"
    repo: str = "httprouter"
    commit: str = "484018016424d215c0b87c42f4c9b57d980fbd00"


@dataclass
class Httptreemux(GoProfile):
    owner: str = "dimfeld"
    repo: str = "httptreemux"
    commit: str = "53a6a09954e8593e66a0c372335c0e96b318b920"


# Register all Go profiles with the global registry
for name, obj in list(globals().items()):
    if (
        isinstance(obj, type)
        and issubclass(obj, GoProfile)
        and obj.__name__ != "GoProfile"
    ):
        registry.register_profile(obj)



from abc import ABC
from swesmith.bug_gen.procedural.base import ProceduralModifier


class GolangProceduralModifier(ProceduralModifier, ABC):
    """Base class for Go-specific procedural modifications."""



import libcst

from swesmith.bug_gen.procedural.base import CommonPMs
from swesmith.bug_gen.procedural.python.base import PythonProceduralModifier
from swesmith.constants import CodeProperty


class RemoveLoopModifier(PythonProceduralModifier):
    explanation: str = CommonPMs.REMOVE_LOOP.explanation
    name: str = CommonPMs.REMOVE_LOOP.name
    conditions: list = CommonPMs.REMOVE_LOOP.conditions

    class Transformer(PythonProceduralModifier.Transformer):
        def leave_For(self, original_node, updated_node):
            return libcst.RemoveFromParent() if self.flip() else updated_node

        def leave_While(self, original_node, updated_node):
            return libcst.RemoveFromParent() if self.flip() else updated_node


class RemoveConditionalModifier(PythonProceduralModifier):
    explanation: str = CommonPMs.REMOVE_CONDITIONAL.explanation
    name: str = CommonPMs.REMOVE_CONDITIONAL.name
    conditions: list = CommonPMs.REMOVE_CONDITIONAL.conditions

    class Transformer(PythonProceduralModifier.Transformer):
        def leave_If(self, original_node, updated_node):
            return libcst.RemoveFromParent() if self.flip() else updated_node


class RemoveAssignModifier(PythonProceduralModifier):
    explanation: str = CommonPMs.REMOVE_ASSIGNMENT.explanation
    name: str = CommonPMs.REMOVE_ASSIGNMENT.name
    conditions: list = CommonPMs.REMOVE_ASSIGNMENT.conditions

    class Transformer(PythonProceduralModifier.Transformer):
        def leave_Assign(self, original_node, updated_node):
            return libcst.RemoveFromParent() if self.flip() else updated_node

        def leave_AugAssign(self, original_node, updated_node):
            return libcst.RemoveFromParent() if self.flip() else updated_node


class RemoveWrapperModifier(PythonProceduralModifier):
    explanation: str = "There are missing wrappers (with, try blocks) in the code."
    name: str = "func_pm_remove_wrapper"
    conditions: list = [
        CodeProperty.IS_FUNCTION,
        CodeProperty.HAS_WRAPPER,
    ]

    class Transformer(PythonProceduralModifier.Transformer):
        def leave_With(self, original_node, updated_node):
            return libcst.RemoveFromParent() if self.flip() else updated_node

        def leave_AsyncWith(self, original_node, updated_node):
            return libcst.RemoveFromParent() if self.flip() else updated_node

        def leave_Try(self, original_node, updated_node):
            return libcst.RemoveFromParent() if self.flip() else updated_node



import libcst

from abc import ABC
from swesmith.bug_gen.procedural.base import ProceduralModifier
from swesmith.constants import BugRewrite, CodeEntity


class PythonProceduralModifier(ProceduralModifier, ABC):
    """Base class for Python-specific procedural modifications using LibCST."""

    class Transformer(libcst.CSTTransformer):
        """Nested LibCST transformer that has access to parent modifier."""

        def __init__(self, parent_modifier):
            self.parent = parent_modifier
            super().__init__()

        def flip(self) -> bool:
            """Delegate to parent's flip method."""
            return self.parent.flip()

    def modify(self, code_entity: CodeEntity) -> BugRewrite | None:
        try:
            module = libcst.parse_module(code_entity.src_code)
        except libcst.ParserSyntaxError:
            # Failed to parse code - syntax errors, malformed code, etc.
            return None

        changed = False
        transformer = self.Transformer(self)

        try:
            for _ in range(self.max_attempts):
                modified = module.visit(transformer)
                if module.code != modified.code:
                    changed = True
                    break
        except (AttributeError, TypeError, ValueError):
            return None

        if not changed:
            return None

        return BugRewrite(
            rewrite=modified.code,
            explanation=self.explanation,
            cost=0.0,
            strategy=self.name,
        )



"""From: https://github.com/SWE-Gym/SWE-Gym/blob/main/scripts/training/openhands/train_torchtune_full.py

Full fine tune an LM using torchtune

modal run swesmith/train/run/ft_torchtune.py --config /path/to/config.yaml
"""

import os
import modal
import yaml

torchtune_image = (
    modal.Image.debian_slim(python_version="3.12")
    .apt_install("git")
    .pip_install(
        [
            "torch",
            "torchvision",
            "torchao",
            "wandb",
            "torchtune",
        ]
    )
)


app = modal.App("torchtune-training")
trained_model_volume = modal.Volume.from_name("weights", create_if_missing=True)
dataset_volume = modal.Volume.from_name("data", create_if_missing=True)

MINUTES = 60  # seconds
HOURS = 60 * MINUTES
N_GPUS = int(os.environ.get("N_GPUS", 2))
N_HOURS = int(os.environ.get("N_HOURS", 10))


@app.function(
    image=torchtune_image,
    # gpu=modal.gpu.A100(count=N_GPU, size="80GB"),
    gpu=f"H100:{N_GPUS}",
    volumes={
        "/weights": trained_model_volume,
        "/data": dataset_volume,
    },
    timeout=N_HOURS * HOURS,
    secrets=[
        modal.Secret.from_name("john-wandb-secret"),
        modal.Secret.from_name("john-hf-secret"),
    ],
)
def run_train(config_name: str, config: dict, n_gpus: int):
    config_path = f"/tmp/{config_name}.yaml"
    with open(config_path, "w") as f:
        yaml.dump(config, f)
    command = f"tune run --nnodes 1 --nproc_per_node {n_gpus} full_finetune_distributed --config {config_path}"
    import subprocess
    import sys

    subprocess.run(
        command.split(),
        stdout=sys.stdout,
        stderr=sys.stderr,
        check=True,
    )
    trained_model_volume.commit()


@app.local_entrypoint()
def main(config: str):
    # load yaml config
    config_name = os.path.basename(config)
    with open(config, "r") as f:
        _config = yaml.safe_load(f)
    run_train.remote(config_name=config_name, config=_config, n_gpus=N_GPUS)



import tree_sitter_rust as tsrs

from swesmith.bug_gen.procedural.base import CommonPMs
from swesmith.bug_gen.procedural.rust.base import RustProceduralModifier
from swesmith.constants import BugRewrite, CodeEntity
from tree_sitter import Language, Parser

RUST_LANGUAGE = Language(tsrs.language())

ALL_BINARY_OPERATORS = [
    "+",
    "-",
    "*",
    "/",
    "%",
    "<<",
    ">>",
    "&",
    "|",
    "^",
    "==",
    "!=",
    "<",
    "<=",
    ">",
    ">=",
    "&&",
    "||",
]

FLIPPED_OPERATORS = {
    "+": "-",
    "-": "+",
    "*": "/",
    "/": "*",
    "%": "*",
    "<<": ">>",
    ">>": "<<",
    "&": "|",
    "|": "&",
    "^": "&",
    "==": "!=",
    "!=": "==",
    "<": ">",
    "<=": ">=",
    ">": "<",
    ">=": "<=",
    "&&": "||",
    "||": "&&",
}

# Operator groups for systematic changes
ARITHMETIC_OPS = ["+", "-", "*", "/", "%"]
BITWISE_OPS = ["&", "|", "^", "<<", ">>"]
COMPARISON_OPS = ["==", "!=", "<", "<=", ">", ">="]
LOGICAL_OPS = ["&&", "||"]

ALL_BINARY_OPERATORS = [
    "+",
    "-",
    "*",
    "/",
    "%",
    "<<",
    ">>",
    "&",
    "|",
    "^",
    "==",
    "!=",
    "<",
    "<=",
    ">",
    ">=",
    "&&",
    "||",
]


class OperationChangeModifier(RustProceduralModifier):
    explanation: str = CommonPMs.OPERATION_CHANGE.explanation
    name: str = CommonPMs.OPERATION_CHANGE.name
    conditions: list = CommonPMs.OPERATION_CHANGE.conditions

    def modify(self, code_entity: CodeEntity) -> BugRewrite:
        """Apply operation changes to Rust binary expressions."""
        if not self.flip():
            return None

        parser = Parser(RUST_LANGUAGE)
        tree = parser.parse(bytes(code_entity.src_code, "utf8"))

        modified_code = self._change_operations(code_entity.src_code, tree.root_node)

        if modified_code == code_entity.src_code:
            return None

        return BugRewrite(
            rewrite=modified_code,
            explanation=self.explanation,
            strategy=self.name,
        )

    def _change_operations(self, source_code: str, node) -> str:
        """Recursively find and change binary operations."""
        modifications = []

        def collect_binary_ops(n):
            if n.type == "binary_expression":
                operator_node = None
                for child in n.children:
                    if child.type in ALL_BINARY_OPERATORS:
                        operator_node = child
                        break

                if operator_node and self.flip():
                    op = operator_node.text.decode("utf-8")
                    new_op = self._get_alternative_operator(op)
                    if new_op != op:
                        modifications.append((operator_node, new_op))

            for child in n.children:
                collect_binary_ops(child)

        collect_binary_ops(node)

        modified_code = source_code
        for operator_node, new_op in sorted(
            modifications, key=lambda x: x[0].start_byte, reverse=True
        ):
            start_byte = operator_node.start_byte
            end_byte = operator_node.end_byte
            modified_code = (
                modified_code[:start_byte] + new_op + modified_code[end_byte:]
            )

        return modified_code

    def _get_alternative_operator(self, op: str) -> str:
        """Get an alternative operator from the same category."""
        if op in ARITHMETIC_OPS:
            return self.rand.choice(ARITHMETIC_OPS)
        elif op in BITWISE_OPS:
            return self.rand.choice(BITWISE_OPS)
        elif op in COMPARISON_OPS:
            return self.rand.choice(COMPARISON_OPS)
        elif op in LOGICAL_OPS:
            return self.rand.choice(LOGICAL_OPS)
        return op


class OperationFlipOperatorModifier(RustProceduralModifier):
    explanation: str = CommonPMs.OPERATION_FLIP_OPERATOR.explanation
    name: str = CommonPMs.OPERATION_FLIP_OPERATOR.name
    conditions: list = CommonPMs.OPERATION_FLIP_OPERATOR.conditions

    def modify(self, code_entity: CodeEntity) -> BugRewrite:
        """Apply operator flipping to Rust binary expressions."""
        if not self.flip():
            return None

        parser = Parser(RUST_LANGUAGE)
        tree = parser.parse(bytes(code_entity.src_code, "utf8"))

        modified_code = self._flip_operators(code_entity.src_code, tree.root_node)

        if modified_code == code_entity.src_code:
            return None

        return BugRewrite(
            rewrite=modified_code,
            explanation=self.explanation,
            strategy=self.name,
        )

    def _flip_operators(self, source_code: str, node) -> str:
        """Recursively find and flip binary operations."""
        modifications = []

        def collect_binary_ops(n):
            if n.type == "binary_expression":
                operator_node = None
                left_operand = None

                for i, child in enumerate(n.children):
                    if child.type in FLIPPED_OPERATORS:
                        operator_node = child
                        if i > 0:
                            left_operand = n.children[0]
                        break

                if operator_node and self.flip():
                    op = operator_node.text.decode("utf-8")
                    if op in FLIPPED_OPERATORS:
                        if (
                            op == "*"
                            and left_operand
                            and left_operand.type == "range_expression"
                        ):
                            pass  # Skip this - it's a dereference, not multiplication
                        else:
                            modifications.append((operator_node, FLIPPED_OPERATORS[op]))

            for child in n.children:
                collect_binary_ops(child)

        collect_binary_ops(node)

        modified_code = source_code
        for operator_node, new_op in sorted(
            modifications, key=lambda x: x[0].start_byte, reverse=True
        ):
            start_byte = operator_node.start_byte
            end_byte = operator_node.end_byte
            modified_code = (
                modified_code[:start_byte] + new_op + modified_code[end_byte:]
            )

        return modified_code


class OperationSwapOperandsModifier(RustProceduralModifier):
    explanation: str = CommonPMs.OPERATION_SWAP_OPERANDS.explanation
    name: str = CommonPMs.OPERATION_SWAP_OPERANDS.name
    conditions: list = CommonPMs.OPERATION_SWAP_OPERANDS.conditions

    def modify(self, code_entity: CodeEntity) -> BugRewrite:
        """Apply operand swapping to Rust binary expressions."""
        if not self.flip():
            return None

        parser = Parser(RUST_LANGUAGE)
        tree = parser.parse(bytes(code_entity.src_code, "utf8"))

        modified_code = self._swap_operands(code_entity.src_code, tree.root_node)

        if modified_code == code_entity.src_code:
            return None

        return BugRewrite(
            rewrite=modified_code,
            explanation=self.explanation,
            strategy=self.name,
        )

    def _swap_operands(self, source_code: str, node) -> str:
        """Recursively find and swap operands in binary operations."""
        modifications = []

        def collect_binary_ops(n):
            if n.type == "binary_expression" and len(n.children) >= 3:
                if self.flip():
                    left_operand = n.children[0]
                    operator = None
                    right_operand = None

                    for i, child in enumerate(n.children[1:], 1):
                        if child.type in ALL_BINARY_OPERATORS:
                            operator = child
                            if i + 1 < len(n.children):
                                right_operand = n.children[i + 1]
                            break

                    if left_operand and operator and right_operand:
                        modifications.append((n, left_operand, operator, right_operand))

            for child in n.children:
                collect_binary_ops(child)

        collect_binary_ops(node)

        modified_code = source_code
        for expr_node, left, op, right in sorted(
            modifications, key=lambda x: x[0].start_byte, reverse=True
        ):
            start_byte = expr_node.start_byte
            end_byte = expr_node.end_byte

            left_text = left.text.decode("utf-8")
            op_text = op.text.decode("utf-8")
            right_text = right.text.decode("utf-8")

            new_expr = f"{right_text} {op_text} {left_text}"
            modified_code = (
                modified_code[:start_byte] + new_expr + modified_code[end_byte:]
            )

        return modified_code


class OperationBreakChainsModifier(RustProceduralModifier):
    explanation: str = CommonPMs.OPERATION_BREAK_CHAINS.explanation
    name: str = CommonPMs.OPERATION_BREAK_CHAINS.name
    conditions: list = CommonPMs.OPERATION_BREAK_CHAINS.conditions

    def modify(self, code_entity: CodeEntity) -> BugRewrite:
        """Apply chain breaking to Rust binary expressions."""
        if not self.flip():
            return None

        parser = Parser(RUST_LANGUAGE)
        tree = parser.parse(bytes(code_entity.src_code, "utf8"))

        modified_code = self._break_chains(code_entity.src_code, tree.root_node)

        if modified_code == code_entity.src_code:
            return None

        return BugRewrite(
            rewrite=modified_code,
            explanation=self.explanation,
            strategy=self.name,
        )

    def _break_chains(self, source_code: str, node) -> str:
        """Recursively find and break chains in binary operations."""
        modifications = []

        def collect_binary_ops(n):
            if n.type == "binary_expression" and self.flip():
                left_operand = n.children[0] if n.children else None
                right_operand = None

                for i, child in enumerate(n.children[1:], 1):
                    if child.type not in ALL_BINARY_OPERATORS:
                        right_operand = child
                        break

                if left_operand and left_operand.type == "binary_expression":
                    inner_left = (
                        left_operand.children[0] if left_operand.children else None
                    )
                    if inner_left:
                        modifications.append((n, inner_left))
                elif right_operand and right_operand.type == "binary_expression":
                    inner_right = None
                    for child in reversed(right_operand.children):
                        if child.type not in ALL_BINARY_OPERATORS:
                            inner_right = child
                            break
                    if inner_right:
                        modifications.append((n, inner_right))

            for child in n.children:
                collect_binary_ops(child)

        collect_binary_ops(node)

        modified_code = source_code
        for expr_node, replacement in sorted(
            modifications, key=lambda x: x[0].start_byte, reverse=True
        ):
            start_byte = expr_node.start_byte
            end_byte = expr_node.end_byte
            replacement_text = replacement.text.decode("utf-8")
            modified_code = (
                modified_code[:start_byte] + replacement_text + modified_code[end_byte:]
            )

        return modified_code


class OperationChangeConstantsModifier(RustProceduralModifier):
    explanation: str = CommonPMs.OPERATION_CHANGE_CONSTANTS.explanation
    name: str = CommonPMs.OPERATION_CHANGE_CONSTANTS.name
    conditions: list = CommonPMs.OPERATION_CHANGE_CONSTANTS.conditions

    def modify(self, code_entity: CodeEntity) -> BugRewrite:
        """Apply constant changes to Rust binary expressions."""
        if not self.flip():
            return None

        parser = Parser(RUST_LANGUAGE)
        tree = parser.parse(bytes(code_entity.src_code, "utf8"))

        modified_code = self._change_constants(code_entity.src_code, tree.root_node)

        if modified_code == code_entity.src_code:
            return None

        return BugRewrite(
            rewrite=modified_code,
            explanation=self.explanation,
            strategy=self.name,
        )

    def _change_constants(self, source_code: str, node) -> str:
        """Recursively find and modify constants in binary operations."""
        modifications = []

        def collect_binary_ops(n):
            if n.type == "binary_expression":
                for child in n.children:
                    if child.type == "integer_literal" and self.flip():
                        try:
                            value = int(child.text.decode("utf-8"))
                            new_value = value + self.rand.choice([-1, 1])
                            modifications.append((child, str(new_value)))
                        except ValueError:
                            pass
                    elif child.type == "float_literal" and self.flip():
                        try:
                            value = float(child.text.decode("utf-8"))
                            delta = self.rand.choice([-0.1, 0.1, -1.0, 1.0])
                            new_value = value + delta
                            modifications.append((child, str(new_value)))
                        except ValueError:
                            pass

            for child in n.children:
                collect_binary_ops(child)

        collect_binary_ops(node)

        modified_code = source_code
        for const_node, new_value in sorted(
            modifications, key=lambda x: x[0].start_byte, reverse=True
        ):
            start_byte = const_node.start_byte
            end_byte = const_node.end_byte
            modified_code = (
                modified_code[:start_byte] + new_value + modified_code[end_byte:]
            )

        return modified_code



"""
Common pytest fixtures and configuration for SWE-smith tests.
"""

import os
import pytest
import sys

from pathlib import Path


# Add the repository root to the Python path to ensure imports work correctly
repo_root = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))
if repo_root not in sys.path:
    sys.path.insert(0, repo_root)


@pytest.fixture
def test_file_c():
    return Path(repo_root) / "tests/test_logs/files/c/tini.c"


@pytest.fixture
def test_file_cpp():
    return Path(repo_root) / "tests/test_logs/files/cpp/util.cpp"


@pytest.fixture
def test_file_c_sharp():
    return Path(repo_root) / "tests/test_logs/files/c_sharp/ReadTraceNexusImporter.cs"


@pytest.fixture
def test_file_go_caddy_listeners():
    return Path(repo_root) / "tests/test_logs/files/go/caddy/listeners.go"


@pytest.fixture
def test_file_go_caddy_usagepool():
    return Path(repo_root) / "tests/test_logs/files/go/caddy/usagepool.go"


@pytest.fixture
def test_file_go_gin():
    return Path(repo_root) / "tests/test_logs/files/go/gin/logger.go"


@pytest.fixture
def test_file_php():
    return Path(repo_root) / "tests/test_logs/files/php/ControllerDispatcher.php"


@pytest.fixture
def test_file_java():
    return Path(repo_root) / "tests/test_logs/files/java/InOrderImpl.java"


@pytest.fixture
def test_file_py():
    return Path(repo_root) / "tests/test_logs/files/python/extension.py"


@pytest.fixture
def test_file_ruby():
    return Path(repo_root) / "tests/test_logs/files/ruby/query_parser.rb"


@pytest.fixture
def test_file_rust():
    return Path(repo_root) / "tests/test_logs/files/rust/cookie.rs"


@pytest.fixture
def test_file_js():
    return Path(repo_root) / "tests/test_logs/files/javascript/sample.js"


@pytest.fixture
def test_output_gotest():
    return (
        Path(repo_root)
        / "tests/test_logs/test_output/gin-gonic__gin.3c12d2a8.lm_rewrite__4pb48n1g.txt"
    )


@pytest.fixture
def test_output_pytest():
    return (
        Path(repo_root)
        / "tests/test_logs/test_output/django-money__django-money.835c1ab8.combine_file__7znr0kum.txt"
    )


@pytest.fixture
def logs_trajectories():
    return Path(repo_root) / "tests/test_logs/trajectories"


@pytest.fixture
def logs_run_evaluation():
    return Path(repo_root) / "tests/test_logs/run_evaluation"


@pytest.fixture
def logs_run_validation():
    return Path(repo_root) / "tests/test_logs/run_validation"


@pytest.fixture
def ft_xml_example():
    return Path(repo_root) / "tests/test_logs/run_evaluation.xml.jsonl"


@pytest.fixture
def task_instance_path():
    return Path(repo_root) / "tests/test_logs/pandas-dev__pandas.95280573.pr_53652.json"



"""
JavaScript control flow modifiers for procedural bug generation using tree-sitter.
"""

import tree_sitter_javascript as tsjs
from swesmith.bug_gen.procedural.base import CommonPMs
from swesmith.bug_gen.procedural.javascript.base import JavaScriptProceduralModifier
from swesmith.constants import BugRewrite, CodeEntity
from tree_sitter import Language, Parser

JS_LANGUAGE = Language(tsjs.language())


class ControlIfElseInvertModifier(JavaScriptProceduralModifier):
    """Invert if-else blocks by swapping their bodies"""

    explanation: str = CommonPMs.CONTROL_IF_ELSE_INVERT.explanation
    name: str = CommonPMs.CONTROL_IF_ELSE_INVERT.name
    conditions: list = CommonPMs.CONTROL_IF_ELSE_INVERT.conditions
    min_complexity: int = 5

    def modify(self, code_entity: CodeEntity) -> BugRewrite:
        """Swap if and else blocks."""
        # Parse the code
        parser = Parser(JS_LANGUAGE)
        tree = parser.parse(bytes(code_entity.src_code, "utf8"))

        changed = False
        for _ in range(self.max_attempts):
            modified_code = self._invert_if_else_statements(
                code_entity.src_code, tree.root_node
            )

            if modified_code != code_entity.src_code:
                changed = True
                break

        if not changed:
            return None

        return BugRewrite(
            rewrite=modified_code,
            explanation=self.explanation,
            strategy=self.name,
        )

    def _invert_if_else_statements(self, source_code: str, node) -> str:
        """Recursively find and invert if-else statements by swapping the bodies."""
        modifications = []
        source_bytes = source_code.encode("utf-8")

        def collect_if_statements(n):
            if n.type == "if_statement":
                # Parse the if statement structure
                # JavaScript if statement: if (condition) consequence [else alternative]
                condition = None
                consequence = None
                alternative = None

                for i, child in enumerate(n.children):
                    if child.type == "if":
                        continue  # Skip the "if" keyword
                    elif child.type == "parenthesized_expression":
                        condition = child
                    elif child.type == "statement_block" and consequence is None:
                        consequence = child  # First block is the if body
                    elif child.type == "else_clause":
                        # The else clause contains the alternative
                        for else_child in child.children:
                            if else_child.type == "statement_block":
                                alternative = else_child
                                break
                        break

                # Only modify if we have a complete if-else structure
                if condition and consequence and alternative and self.flip():
                    modifications.append(
                        {
                            "node": n,
                            "condition": condition,
                            "consequence": consequence,
                            "alternative": alternative,
                        }
                    )

            for child in n.children:
                collect_if_statements(child)

        collect_if_statements(node)

        if not modifications:
            return source_code

        # Work with bytes for modifications
        modified_source = source_bytes
        for mod in reversed(modifications):
            node = mod["node"]
            condition = mod["condition"]
            consequence = mod["consequence"]
            alternative = mod["alternative"]

            # Get the text of each part
            condition_text = source_bytes[
                condition.start_byte : condition.end_byte
            ].decode("utf-8")
            consequence_text = source_bytes[
                consequence.start_byte : consequence.end_byte
            ].decode("utf-8")
            alternative_text = source_bytes[
                alternative.start_byte : alternative.end_byte
            ].decode("utf-8")

            # Build the inverted if-else statement
            inverted = f"if {condition_text} {alternative_text} else {consequence_text}"

            # Replace the entire if statement
            modified_source = (
                modified_source[: node.start_byte]
                + inverted.encode("utf-8")
                + modified_source[node.end_byte :]
            )

        return modified_source.decode("utf-8")


class ControlShuffleLinesModifier(JavaScriptProceduralModifier):
    """Shuffle independent statements within a function body"""

    explanation: str = CommonPMs.CONTROL_SHUFFLE_LINES.explanation
    name: str = CommonPMs.CONTROL_SHUFFLE_LINES.name
    conditions: list = CommonPMs.CONTROL_SHUFFLE_LINES.conditions
    max_complexity: int = 10

    def modify(self, code_entity: CodeEntity) -> BugRewrite:
        """Shuffle statements within a function body."""
        # Parse the code
        parser = Parser(JS_LANGUAGE)
        tree = parser.parse(bytes(code_entity.src_code, "utf8"))

        # Find function body and shuffle statements
        modified_code = self._shuffle_statements(code_entity.src_code, tree.root_node)

        if modified_code == code_entity.src_code:
            return None

        return BugRewrite(
            rewrite=modified_code,
            explanation=self.explanation,
            strategy=self.name,
        )

    def _shuffle_statements(self, source_code: str, node) -> str:
        """Find function bodies and shuffle their statements."""
        shuffles = []
        source_bytes = source_code.encode("utf-8")

        def collect_function_bodies(n):
            # Look for function bodies (statement blocks inside functions)
            if n.type in [
                "function_declaration",
                "function_expression",
                "arrow_function",
                "method_definition",
            ]:
                # Find the body
                for child in n.children:
                    if child.type == "statement_block":
                        # Get all direct statement children
                        statements = [
                            c
                            for c in child.children
                            if c.type not in ["{", "}", "\n"]
                            and c.type.endswith("statement")
                        ]

                        # Only shuffle if we have multiple statements
                        if len(statements) >= 2 and self.flip():
                            shuffles.append(
                                {"block": child, "statements": statements.copy()}
                            )
                        return  # Don't recurse into nested functions

            for child in n.children:
                collect_function_bodies(child)

        collect_function_bodies(node)

        if not shuffles:
            return source_code

        # Work with bytes for modifications
        modified_source = source_bytes
        for shuffle_info in reversed(shuffles):
            block = shuffle_info["block"]
            statements = shuffle_info["statements"]

            # Shuffle the statements
            self.rand.shuffle(statements)

            # Build new block content
            new_statements = []
            for stmt in statements:
                stmt_text = source_bytes[stmt.start_byte : stmt.end_byte].decode(
                    "utf-8"
                )
                new_statements.append(stmt_text)

            # Find the opening and closing braces
            block_start = block.start_byte
            block_end = block.end_byte

            # Get indentation from first statement
            first_stmt_start = statements[0].start_byte
            indent_start = first_stmt_start
            while indent_start > block_start and source_bytes[indent_start - 1] in [
                ord(" "),
                ord("\t"),
            ]:
                indent_start -= 1

            indent = source_bytes[indent_start:first_stmt_start].decode("utf-8")

            # Build new block
            new_block = "{\n" + indent + f"\n{indent}".join(new_statements) + "\n}"

            # Replace the block
            modified_source = (
                modified_source[:block_start]
                + new_block.encode("utf-8")
                + modified_source[block_end:]
            )

        return modified_source.decode("utf-8")



import re

from dataclasses import dataclass, field
from swebench.harness.constants import TestStatus
from swesmith.constants import ENV_NAME
from swesmith.profiles.base import RepoProfile, registry


@dataclass
class JavaProfile(RepoProfile):
    """
    Profile for Java repositories.
    """

    exts: list[str] = field(default_factory=lambda: [".java"])


@dataclass
class Gsondd2fe59c(JavaProfile):
    owner: str = "google"
    repo: str = "gson"
    commit: str = "dd2fe59c0d3390b2ad3dd365ed6938a5c15844cb"
    test_cmd: str = "mvn test -B -T 1C -Dsurefire.useFile=false -Dsurefire.printSummary=true -Dsurefire.reportFormat=plain"
    eval_sets: set[str] = field(
        default_factory=lambda: {"SWE-bench/SWE-bench_Multilingual"}
    )

    @property
    def dockerfile(self):
        return f"""FROM ubuntu:22.04
ENV DEBIAN_FRONTEND=noninteractive
ENV LANG=C.UTF-8
ENV LC_ALL=C.UTF-8
RUN apt-get update && apt-get install -y git openjdk-11-jdk
RUN apt-get install -y maven
RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN mvn clean install -B -pl gson -DskipTests -am
"""

    def log_parser(self, log: str) -> dict[str, str]:
        test_status_map = {}
        pattern = r"^\[(INFO|ERROR)\]\s+(.*?)\s+--\s+Time elapsed:\s+([\d.]+)\s"
        for line in log.split("\n"):
            if line.endswith("<<< FAILURE!") and line.startswith("[ERROR]"):
                test_name = re.match(pattern, line)
                if test_name is None:
                    continue
                test_status_map[test_name.group(2)] = TestStatus.FAILED.value
            elif (
                any([line.startswith(s) for s in ["[INFO]", "[ERROR]"]])
                and "Time elapsed:" in line
            ):
                test_name = re.match(pattern, line)
                if test_name is None:
                    continue
                test_status_map[test_name.group(2)] = TestStatus.PASSED.value
        return test_status_map


# Register all Java profiles with the global registry
for name, obj in list(globals().items()):
    if (
        isinstance(obj, type)
        and issubclass(obj, JavaProfile)
        and obj.__name__ != "JavaProfile"
    ):
        registry.register_profile(obj)



import libcst

from swesmith.bug_gen.procedural.base import CommonPMs
from swesmith.bug_gen.procedural.python.base import PythonProceduralModifier


class ControlIfElseInvertModifier(PythonProceduralModifier):
    explanation: str = CommonPMs.CONTROL_IF_ELSE_INVERT.explanation
    name: str = CommonPMs.CONTROL_IF_ELSE_INVERT.name
    conditions: list = CommonPMs.CONTROL_IF_ELSE_INVERT.conditions
    min_complexity: int = 5

    class Transformer(PythonProceduralModifier.Transformer):
        def leave_If(
            self, original_node: libcst.If, updated_node: libcst.If
        ) -> libcst.If:
            if not self.flip():
                return updated_node

            # Only proceed if there's an else branch to swap with
            if not updated_node.orelse:
                return updated_node

            # We need to handle standard else blocks
            if isinstance(updated_node.orelse, libcst.Else):
                # Store the original bodies
                if_body = updated_node.body
                else_body = updated_node.orelse.body

                # Create a new else clause with the original if body
                new_else = libcst.Else(
                    body=if_body,
                    whitespace_before_colon=updated_node.orelse.whitespace_before_colon,
                )

                # Return a new If statement with swapped bodies
                return updated_node.with_changes(body=else_body, orelse=new_else)

            # Skip elif cases for now
            return updated_node


class ControlShuffleLinesModifier(PythonProceduralModifier):
    explanation: str = CommonPMs.CONTROL_SHUFFLE_LINES.explanation
    name: str = CommonPMs.CONTROL_SHUFFLE_LINES.name
    conditions: list = CommonPMs.CONTROL_SHUFFLE_LINES.conditions
    max_complexity: int = 10

    class Transformer(PythonProceduralModifier.Transformer):
        def leave_FunctionDef(
            self, original_node: libcst.FunctionDef, updated_node: libcst.FunctionDef
        ) -> libcst.FunctionDef:
            # Skip modification if random check fails
            if not self.flip():
                return updated_node

            # Make sure we're working with an indented block
            if not isinstance(updated_node.body, libcst.IndentedBlock):
                return updated_node

            # Get the body statements
            body = list(updated_node.body.body)

            # Don't shuffle if there are fewer than 2 statements
            if len(body) < 2:
                return updated_node

            # Create a shuffled copy of the statements
            shuffled_body = body.copy()
            self.parent.rand.shuffle(shuffled_body)

            # Create a new indented block with the shuffled statements
            new_body = libcst.IndentedBlock(
                body=tuple(shuffled_body),
                indent=updated_node.body.indent,
                header=updated_node.body.header,
                footer=updated_node.body.footer,
            )

            # Return the updated function with the new body
            return updated_node.with_changes(body=new_body)



import docker
import fnmatch
import traceback

from concurrent.futures import ThreadPoolExecutor, as_completed
from docker.models.containers import Container
from logging import Logger
from pathlib import Path
from swebench.harness.constants import (
    APPLY_PATCH_FAIL,
    APPLY_PATCH_PASS,
    DOCKER_PATCH,
    DOCKER_USER,
    DOCKER_WORKDIR,
    KEY_INSTANCE_ID,
    LOG_INSTANCE,
    LOG_TEST_OUTPUT,
    RUN_EVALUATION_LOG_DIR,
    TESTS_TIMEOUT,
    UTF8,
)
from swebench.harness.docker_build import setup_logger
from swebench.harness.docker_utils import (
    cleanup_container,
    copy_to_container,
    exec_run_with_timeout,
)
from swebench.harness.utils import EvaluationError
from swesmith.constants import (
    GIT_APPLY_CMDS,
    LOG_DIR_RUN_VALIDATION,
    TEST_OUTPUT_END,
    TEST_OUTPUT_START,
)
from swesmith.profiles import registry
from unidiff import PatchSet


def matches_instance_filter(instance_id: str, instance_ids: list[str] | None) -> bool:
    """
    Check if an instance_id matches the filtering criteria.

    Args:
        instance_id: The instance ID to check
        instance_ids: List of instance IDs or patterns to match against

    Returns:
        True if the instance should be included, False otherwise
    """
    if instance_ids is None:
        return True

    for filter_item in instance_ids:
        # Check for exact match first
        if instance_id == filter_item:
            return True

        # Check for pattern match (supports * and ? wildcards)
        if fnmatch.fnmatch(instance_id, filter_item):
            return True

    return False


def _apply_patch(
    instance_id: str, container: Container, logger: Logger, is_gold: bool = False
):
    """
    Apply a patch to a container's codebase
    """
    apply_succeeded = False
    for git_apply_cmd in GIT_APPLY_CMDS:
        # Because gold patches = bug patches, so fix = revert
        git_apply_cmd = (
            f"{git_apply_cmd} {DOCKER_PATCH}"
            if not is_gold
            else f"{git_apply_cmd} --reverse {DOCKER_PATCH}"
        )
        val = container.exec_run(
            git_apply_cmd, workdir=DOCKER_WORKDIR, user=DOCKER_USER
        )
        if val.exit_code == 0:
            apply_succeeded = True
            logger.info(f"{APPLY_PATCH_PASS}:\n{val.output.decode(UTF8)}")
            break
        logger.info(
            f"Failed to apply patch to container with {git_apply_cmd}.\n"
            + f"Error Message: {val.output.decode(UTF8)}\nTrying again..."
        )
    if not apply_succeeded:
        apply_failed_msg = f"{APPLY_PATCH_FAIL}:\n{val.output.decode(UTF8)}"
        logger.info(apply_failed_msg)
        raise EvaluationError(instance_id, apply_failed_msg, logger)


def run_patch_in_container(
    instance: dict,
    run_id: str,
    log_dir: Path,
    timeout: int,
    patch: str | None = None,
    commit: str | None = None,
    f2p_only: bool = False,
    is_gold: bool = False,
) -> tuple[Logger, bool] | None:
    """
    Run a patch in a container. The general logical flow is as follows:
    1. Setup logging directory
    2. Start docker container
    3. Copy patch to container, if provided
        a. Apply patch to codebase
    4. Copy eval script to container
    5. Run eval script, write outputs to logs

    Returns:
        tuple[Logger, bool]: logger and whether the container timed out or None if an error occurred
    """
    container = None
    client = docker.from_env()
    instance_id = instance[KEY_INSTANCE_ID]
    rp = registry.get_from_inst(instance)
    is_eval = log_dir == RUN_EVALUATION_LOG_DIR
    try:
        container_type = None
        if is_eval:
            container_type = "eval"
        elif log_dir == LOG_DIR_RUN_VALIDATION:
            container_type = "val"

        # Setup logging directory
        log_dir = log_dir / run_id / instance_id
        log_dir.mkdir(parents=True, exist_ok=True)
        container_name = f"swesmith.{container_type}.{run_id}.{instance_id}"
        log_file = log_dir / LOG_INSTANCE
        logger = setup_logger(container_name, log_file)

        # Start docker container
        rp.pull_image()
        container = client.containers.create(
            image=rp.image_name,
            name=container_name,
            user=DOCKER_USER,
            detach=True,
            command="tail -f /dev/null",
            platform="linux/x86_64",
            mem_limit="10g",
        )
        container.start()

        # If provided, checkout commit in container
        if commit is not None:
            logger.info(f"Checking out commit {commit}")
            container.exec_run("git fetch", workdir=DOCKER_WORKDIR, user=DOCKER_USER)
            val = container.exec_run(
                f"git checkout {commit}", workdir=DOCKER_WORKDIR, user=DOCKER_USER
            )
            if val.exit_code != 0:
                logger.info(f"CHECKOUT FAILED: {val.output.decode(UTF8)}")
                return logger, False
            if is_eval:
                # NOTE: Key assumption we make is that each branch has two commits
                # 1. Bug commit
                # 2. F2P Test File(s) removal commit (on top of 1).
                # The `HEAD~1` corresponds to reverting the branch to (1), which
                # effectively brings the tests back into the codebase.
                val = container.exec_run(
                    "git checkout HEAD~1", workdir=DOCKER_WORKDIR, user=DOCKER_USER
                )
                if val.exit_code != 0:
                    logger.info(
                        f"CHECKOUT TO BUG STAGE FAILED: {val.output.decode(UTF8)}"
                    )
                    return logger, False

        # If provided, copy patch to container and apply it to codebase
        if patch is not None and len(patch) >= 1:
            logger.info("Applying patch to container...")

            # Revert any changes to those files in the container to ensure a clean state
            changed_files = " ".join([x.path for x in PatchSet(patch)])
            container.exec_run(
                f"git checkout -- {changed_files}",
                workdir=DOCKER_WORKDIR,
                user=DOCKER_USER,
            )

            # Apply the patch inside the container
            patch_file = Path(log_dir / "patch.diff")
            patch_file.write_text(patch)
            logger.info(f"Patch written to {patch_file}, now applying to container...")
            copy_to_container(container, patch_file, Path(DOCKER_PATCH))
            _apply_patch(instance_id, container, logger, is_gold)

            if is_eval:
                # For evaluation, removes any changes to test related files.
                f2p_files, p2p_files = rp.get_test_files(instance)
                test_files = " ".join(f2p_files + p2p_files)
                if test_files:
                    container.exec_run(
                        f"git checkout -- {test_files}",
                        workdir=DOCKER_WORKDIR,
                        user=DOCKER_USER,
                    )
                    logger.info(
                        f"Reverted changes to test files in container: {test_files}"
                    )

        # Copy eval script to container
        eval_file = Path(log_dir / "eval.sh")
        test_command, _ = rp.get_test_cmd(instance, f2p_only=f2p_only)
        eval_file.write_text(
            "\n".join(
                [
                    "#!/bin/bash",
                    "set -uxo pipefail",
                    f"cd {DOCKER_WORKDIR}",
                    f": '{TEST_OUTPUT_START}'",
                    test_command,
                    f": '{TEST_OUTPUT_END}'",
                ]
            )
            + "\n"
        )
        copy_to_container(container, eval_file, Path("/eval.sh"))

        # Run eval script, write outputs to logs
        test_output, timed_out, total_runtime = exec_run_with_timeout(
            container, "/bin/bash /eval.sh", timeout=timeout
        )
        test_output_path = log_dir / LOG_TEST_OUTPUT
        logger.info(f"Test Runtime: {total_runtime:_.2f} seconds")
        with open(test_output_path, "w") as f:
            f.write(test_output)
            if timed_out:
                timeout_error = f"{TESTS_TIMEOUT}: {timeout} seconds exceeded"
                f.write(f"\n\n{timeout_error}")

        logger.info(f"Test output for {instance_id} written to {test_output_path}")
        cleanup_container(client, container, logger)
        return logger, timed_out
    except Exception as e:
        error_msg = (
            f"Error validating {instance_id}: {e}\n"
            f"{traceback.format_exc()}\n"
            f"Check ({logger.log_file}) for more information."
        )
        logger.info(error_msg)
        print(f"Error validating {instance_id}: {e}")

        # Remove instance container + image, close logger
        cleanup_container(client, container, logger)
        return logger, False


def run_threadpool(func, payloads, max_workers):
    """
    Run a function with a list of payloads using ThreadPoolExecutor.

    Args:
        func: Function to run for each payload
        payloads: List of payloads to process
        max_workers: Maximum number of worker threads

    Returns:
        tuple: (succeeded, failed) lists of payloads
    """
    if max_workers <= 0:
        return run_sequential(func, payloads)

    succeeded, failed = [], []
    with ThreadPoolExecutor(max_workers=max_workers) as executor:
        # Create a future for running each instance
        futures = {executor.submit(func, *payload): payload for payload in payloads}
        # Wait for each future to complete
        for future in as_completed(futures):
            try:
                # Check if instance ran successfully
                future.result()
                succeeded.append(futures[future])
            except Exception as e:
                print(f"{type(e)}: {e}")
                traceback.print_exc()
                failed.append(futures[future])

    return succeeded, failed


def run_sequential(func, payloads):
    """
    Run a function with a list of payloads sequentially.

    Args:
        func: Function to run for each payload
        payloads: List of payloads to process

    Returns:
        tuple: (succeeded, failed) lists of payloads
    """
    succeeded, failed = [], []
    for payload in payloads:
        try:
            func(*payload)
            succeeded.append(payload)
        except Exception:
            traceback.print_exc()
            failed.append(payload)

    return succeeded, failed



"""
Purpose: Repo-wide constants
"""

import hashlib
import random
import string

from abc import abstractmethod
from dataclasses import dataclass
from enum import Enum
from pathlib import Path
from typing import Any

DEFAULT_PM_LIKELIHOOD = 0.2
ENV_NAME = "testbed"
HF_DATASET = "SWE-bench/SWE-smith"
INSTANCE_REF = "instance_ref"
KEY_IMAGE_NAME = "image_name"
KEY_PATCH = "patch"
KEY_TIMED_OUT = "timed_out"
LOG_DIR_BUG_GEN = Path("logs/bug_gen")
LOG_DIR_ENV = Path("logs/build_images/env")
LOG_DIR_ISSUE_GEN = Path("logs/issue_gen")
LOG_DIR_RUN_VALIDATION = Path("logs/run_validation")
LOG_DIR_TASKS = Path("logs/task_insts")
LOG_TEST_OUTPUT_PRE_GOLD = "test_output_pre_gold.txt"
MAX_INPUT_TOKENS = 128000
ORG_NAME_DH = "swebench"
ORG_NAME_GH = "swesmith"
PREFIX_BUG = "bug"
PREFIX_METADATA = "metadata"
REF_SUFFIX = ".ref"
TEMP_PATCH = "_temp_patch_swesmith.diff"
TEST_OUTPUT_END = ">>>>> End Test Output"
TEST_OUTPUT_START = ">>>>> Start Test Output"
TODO_REWRITE = "TODO: Implement this function"
UBUNTU_VERSION = "22.04"

GIT_APPLY_CMDS = [
    "git apply --verbose",
    "git apply --verbose --reject",
    "patch --batch --fuzz=5 -p1 -i",
]


class Architecture(str, Enum):
    X86_64 = "x86_64"
    ARM64 = "arm64"


class CodeProperty(Enum):
    # Core entity types
    IS_FUNCTION = "is_function"
    IS_CLASS = "is_class"

    # Control flow
    HAS_EXCEPTION = "has_exception"
    HAS_IF = "has_if"
    HAS_IF_ELSE = "has_if_else"
    HAS_LOOP = "has_loop"
    HAS_SWITCH = "has_switch"  # Added for switch statements

    # Operations
    HAS_ARITHMETIC = "has_arithmetic"
    HAS_ASSIGNMENT = "has_assignment"
    HAS_DECORATOR = "has_decorator"
    HAS_FUNCTION_CALL = "has_function_call"
    HAS_IMPORT = "has_import"
    HAS_LAMBDA = "has_lambda"
    HAS_LIST_COMPREHENSION = "has_list_comprehension"
    HAS_LIST_INDEXING = "has_list_indexing"
    HAS_OFF_BY_ONE = "has_off_by_one"
    HAS_PARENT = "has_parent"
    HAS_RETURN = "has_return"
    HAS_WRAPPER = "has_wrapper"

    # Operations by type
    HAS_BINARY_OP = "has_binary_op"
    HAS_BOOL_OP = "has_bool_op"
    HAS_TERNARY = "has_ternary"
    HAS_UNARY_OP = "has_unary_op"


class CodeEntityMeta(type):
    def __new__(mcs, name, bases, namespace):
        # Create properties for all enum values
        for prop in CodeProperty:
            namespace[prop.value] = property(lambda self, p=prop: p in self._tags)
        return super().__new__(mcs, name, bases, namespace)


@dataclass
class CodeEntity(metaclass=CodeEntityMeta):
    """Data class to hold information about a code entity (e.g. function, class)."""

    file_path: str
    indent_level: int
    indent_size: int
    line_end: int
    line_start: int
    node: Any
    src_code: Any

    def __post_init__(self):
        self._tags: set[CodeProperty] = set()
        self._analyze_properties()

    def _analyze_properties(self):
        """To be implemented by language-specific classes"""
        pass

    @property
    def complexity(self) -> int:
        """Get the complexity of the code entity."""
        return -1  # Default value = no notion of complexity implemented

    @property
    def ext(self) -> str:
        if isinstance(self.file_path, Path):
            self.file_path = str(self.file_path)
        return self.file_path.rsplit(".", 1)[-1].lower()

    @property
    @abstractmethod
    def name(self) -> str:
        """Get the name of the code entity."""
        pass

    @property
    @abstractmethod
    def signature(self) -> str:
        """Get the signature of the code entity."""
        pass

    @property
    @abstractmethod
    def stub(self) -> str:
        """Get stub (code with implementation removed) for the code entity."""
        pass


class BugRewrite:
    cost: float = 0
    explanation: str = ""
    output: str
    rewrite: str
    strategy: str

    def __init__(
        self,
        rewrite: str,
        explanation: str,
        strategy: str,
        cost: float = 0,
        output: str = "",
    ):
        self.rewrite = rewrite
        self.explanation = explanation
        self.cost = cost
        self.strategy = strategy
        self.output = output

    def get_hash(self) -> str:
        """Generates a hash for the bug rewrite."""
        return generate_hash(self.rewrite)

    def to_dict(self) -> dict[str, Any]:
        """Converts the bug rewrite to a dictionary."""
        return {
            "cost": self.cost,
            "explanation": self.explanation,
            "output": self.output,
            "rewrite": self.rewrite,
            "strategy": self.strategy,
        }


def generate_hash(s):
    rng = random.Random(int(hashlib.sha256(s.encode()).hexdigest(), 16))
    return "".join(rng.choice(string.ascii_lowercase + string.digits) for _ in range(8))



"""
Purpose: Test the difficulty rater model

Usage: python train/difficulty_rater/test_rater.py --base_url <base_url>

NOTE: Please make sure the sglang server is running and `difficulty_test.jsonl` is in the same directory.
"""

import argparse
import json
import openai

from tqdm.auto import tqdm


def main(base_url: str):
    with open("difficulty_test.jsonl") as f:
        test_insts = [json.loads(x) for x in f]
    client = openai.Client(base_url=f"{base_url}/v1", api_key="swesmith")
    responses = []

    for inst in tqdm(test_insts):
        answer = inst["messages"][-1]["content"]
        messages = inst["messages"][:-1]

        response = client.chat.completions.create(
            model="gpt-4o",
            messages=messages,
            temperature=0,
            max_tokens=64,
        )
        resp = response.choices[0].message.content.strip()
        pred = resp
        if "\n" in pred:
            pred = pred.split("\n")[0]
        responses.append([pred, answer, resp])

    print(
        f"Accuracy: {round(sum([x[0] == x[1] for x in responses]) / len(responses) * 100, 4)}%"
    )

    sig_diff = 0
    for x in responses:
        if (x[0] == "1-4 hours" and x[1] == "<15 min fix") or (
            x[1] == "1-4 hours" and x[0] == "<15 min fix"
        ):
            sig_diff += 1
    print(f"# of significantly different preds: {sig_diff}")


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("--base_url", type=str, default="http://localhost:8000")
    args = parser.parse_args()
    main(**vars(args))



"""
Purpose: Create difficulty train / test datasets from SWE-bench Verified annotations of task difficulty.

Usage:
python train/difficulty_rater/create_datasets.py

NOTE: Please include the follwing files in the same directory when running this script:
- ensembled_annotations_public.csv
- samples_with_3_annotations_public.csv
"""

import json
import pandas as pd

from collections import Counter
from datasets import load_dataset
from swebench.harness.constants import KEY_INSTANCE_ID

PROMPT_SYSTEM = """Below I have given you information about a GitHub pull request. The information includes
the problem statement describing the bug and the patch representing the changes made that
successfully resolves the issue. Please categorize the difficulty of the original task based
on this information. There are 4 levels of difficulty you can choose from:

* <15 min fix
* 15 min - 1 hour
* 1-4 hours
* >4 hours"""

PROMPT_INSTANCE = """### Input:
**Problem Statement**
{problem_statement}

**Solution Patch**
{patch}

**Response**
"""

if __name__ == "__main__":
    sweb = load_dataset("SWE-bench/SWE-bench")
    sweb_map = {x[KEY_INSTANCE_ID]: x for x in sweb["test"]}
    ensembled = pd.read_csv("ensembled_annotations_public.csv")
    samplesw3 = pd.read_csv("samples_with_3_annotations_public.csv")

    df = ensembled[[KEY_INSTANCE_ID, "difficulty"]]
    test_df = df.sample(frac=0.2, random_state=42)
    train_df = df.drop(test_df.index)
    print(f"Train size: {len(train_df)}, Test size: {len(test_df)}")

    for pair in [
        ("difficulty_train.jsonl", train_df),
        ("difficulty_test.jsonl", test_df),
    ]:
        distribution = []
        with open(pair[0], "w") as f:
            for row in pair[1].itertuples(index=False, name=None):
                inst = sweb_map[row[0]]
                label = row[1]
                if label == ">4 hours":
                    label = "1-4 hours"
                messages = {
                    "messages": [
                        {"role": "system", "content": PROMPT_SYSTEM},
                        {"role": "user", "content": PROMPT_INSTANCE.format(**inst)},
                        {"role": "assistant", "content": label},
                    ]
                }
                distribution.append(label)
                f.write(json.dumps(messages) + "\n")

        print(f"{pair[0]} distribution:")
        for k, v in Counter(distribution).items():
            print(f"* {k}: {v} ({round(v * 100 / len(distribution), 2)}%)")

    with open("difficulty_train.jsonl") as f:
        check = [json.loads(x) for x in f.readlines()]
    print(len(check))
    with open("difficulty_test.jsonl") as f:
        check = [json.loads(x) for x in f.readlines()]
    print(len(check))



from swesmith.bug_gen.procedural.base import ProceduralModifier
from swesmith.bug_gen.procedural.golang.control_flow import (
    ControlIfElseInvertModifier,
    ControlShuffleLinesModifier,
)
from swesmith.bug_gen.procedural.golang.operations import (
    OperationChangeModifier,
    OperationFlipOperatorModifier,
    OperationSwapOperandsModifier,
    OperationBreakChainsModifier,
    OperationChangeConstantsModifier,
)
from swesmith.bug_gen.procedural.golang.remove import (
    RemoveAssignModifier,
    RemoveConditionalModifier,
    RemoveLoopModifier,
)

MODIFIERS_GOLANG: list[ProceduralModifier] = [
    ControlIfElseInvertModifier(likelihood=0.75),
    ControlShuffleLinesModifier(likelihood=0.75),
    RemoveAssignModifier(likelihood=0.25),
    RemoveConditionalModifier(likelihood=0.25),
    RemoveLoopModifier(likelihood=0.25),
    OperationBreakChainsModifier(likelihood=0.4),
    OperationChangeConstantsModifier(likelihood=0.4),
    OperationChangeModifier(likelihood=0.4),
    OperationFlipOperatorModifier(likelihood=0.4),
    OperationSwapOperandsModifier(likelihood=0.4),
]



import re

from dataclasses import dataclass, field
from swebench.harness.constants import TestStatus
from swesmith.constants import ENV_NAME
from swesmith.profiles.base import RepoProfile, registry


@dataclass
class CppProfile(RepoProfile):
    """
    Profile for C++ repositories.
    """

    exts: list[str] = field(default_factory=lambda: [".cpp"])


@dataclass
class Catch29b3f508a(CppProfile):
    owner: str = "catchorg"
    repo: str = "Catch2"
    commit: str = "9b3f508a1b1579f5366cf83d19822cb395f23528"
    test_cmd: str = "cd build && ctest"

    @property
    def dockerfile(self):
        return f"""FROM gcc:12
RUN apt-get update && apt-get install -y \
    libbrotli-dev libcurl4-openssl-dev \
    clang build-essential cmake \
    python3 python3-dev python3-pip

RUN git clone https://github.com/{self.mirror_name} /{ENV_NAME}
WORKDIR /{ENV_NAME}
RUN mkdir build && cd build \
    && cmake .. -DCATCH_DEVELOPMENT_BUILD=ON \
    && make all \
    && ctest"""

    def log_parser(self, log: str) -> dict[str, str]:
        test_status_map = {}
        re_passes = [
            re.compile(r"^-- Performing Test (.+) - Success$", re.IGNORECASE),
            re.compile(
                r"^\d+/\d+ Test\s+#\d+: (.+) \.+\s+ Passed\s+.+$", re.IGNORECASE
            ),
        ]
        re_fails = [
            re.compile(r"^-- Performing Test (.+) - Failed$", re.IGNORECASE),
            re.compile(
                r"^\d+/\d+ Test\s+#\d+: (.+) \.+\*\*\*Failed\s+.+$", re.IGNORECASE
            ),
        ]
        re_skips = [
            re.compile(r"^-- Performing Test (.+) - skipped$", re.IGNORECASE),
        ]

        for line in log.splitlines():
            line = line.strip().lower()
            if not line:
                continue

            for re_pass in re_passes:
                pass_match = re_pass.match(line)
                if pass_match:
                    test = pass_match.group(1)
                    test_status_map[test] = TestStatus.PASSED.value

            for re_fail in re_fails:
                fail_match = re_fail.match(line)
                if fail_match:
                    test = fail_match.group(1)
                    test_status_map[test] = TestStatus.FAILED.value

            for re_skip in re_skips:
                skip_match = re_skip.match(line)
                if skip_match:
                    test = skip_match.group(1)
                    test_status_map[test] = TestStatus.SKIPPED.value

        return test_status_map


# Register all C++ profiles with the global registry
for name, obj in list(globals().items()):
    if (
        isinstance(obj, type)
        and issubclass(obj, CppProfile)
        and obj.__name__ != "CppProfile"
    ):
        registry.register_profile(obj)
